# -*- coding: utf-8 -*-
# vim:fenc=utf-8

# Copyright (C) 2010-2020 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
# Copyright (C) 2010-2016 by Dick Kniep <dick.kniep@lindix.nl>
#
# PyHoca GUI is free software; you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# PyHoca GUI is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program; if not, write to the
# Free Software Foundation, Inc.,
# 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.

import sys
import os

def setup_mswinlog(PROG_NAME):
    if hasattr(sys, 'frozen') and str(sys.frozen) in ("windows_exe", "console_exe", "1", ):
        class MSWin_Logging(object):

            softspace = 0
            _fname = os.path.normpath(os.path.join(os.environ['AppData'], PROG_NAME, '%s.%s.log' % (PROG_NAME, os.getpid())))
            _file = None

            def __init__(self, filemode='a'):
                try:
                    os.mkdir(os.path.dirname(self._fname))
                except:
                    pass
                self._filemode = filemode
                if self._filemode == "w+":
                    for _file in os.listdir(os.path.dirname(self._fname)):
                        try:
                            os.remove(os.path.join(os.path.dirname(self._fname), _file))
                        except WindowsError:
                            pass

            def write(self, text, **kwargs):
                if self._file is None:
                    try:
                        try:
                            os.mkdir(os.path.dirname(self._fname))
                        except:
                            pass
                        self._file = open(self._fname, self._filemode)
                    except:
                        pass
                else:
                    self._file.write(text)
                    self._file.flush()

            def flush(self):
                if self._file is not None:
                    self._file.flush()

        sys.stdout = MSWin_Logging(filemode='w+')
        sys.stderr = MSWin_Logging(filemode='a')
        del MSWin_Logging



# -*- coding: utf-8 -*-
# vim:fenc=utf-8

# Copyright (C) 2010-2020 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
# Copyright (C) 2010-2016 by Dick Kniep <dick.kniep@lindix.nl>
#
# PyHoca GUI is free software; you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# PyHoca GUI is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program; if not, write to the
# Free Software Foundation, Inc.,
# 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.

modules ={}

import os

# Python X2Go
import x2go

# wxPython
import wx

wx_major = int(wx.__version__.split('.')[0])

# PyHoca-GUI modules
from . import basepath

class PyHocaGUI_AboutFrame(wx.Frame):
    """\
    wxWidget displaying an ,,About'' window for this application.

    """
    def __init__(self, _PyHocaGUI, caller=None, about_image=None, icon_name=None, about_what=None, ):
        """\
        About window (constructor).

        @param _PyHocaGUI: main application instance
        @type _PyHocaGUI: C{obj}
        @param caller: unused
        @type caller: C{None}
        @param about_image: full image path for background image of About window
        @type about_image: C{str}
        @param icon_name: icon name for window icon
        @type icon_name: C{str}
        @param about_what: about what is this about window?
        @type about_what: C{str}

        """
        self._PyHocaGUI = _PyHocaGUI
        self._pyhoca_logger = self._PyHocaGUI._pyhoca_logger

        fallback_about_image = 'PyHoca-GUI_about-logo.png'

        if about_image is None:
            about_image = '{appname}_about-logo.png'.format(appname=_PyHocaGUI.appname)

        if not about_image.lower().endswith('.png'):
            about_image = '%s.png' % about_image

        about_image = os.path.expanduser(about_image)

        if os.path.basename(about_image) == about_image:
            about_image = os.path.join(basepath.images_basepath, about_image)

        if not os.path.isfile(about_image):
            about_image = os.path.join(basepath.images_basepath, fallback_about_image)

        if about_what is None:
            about_what = self._PyHocaGUI.appname

        if x2go.X2GOCLIENT_OS == 'Windows':
            wx.Frame.__init__(self, None, -1, _('About %s ...') % about_what, size=(403,340))
        else:
            wx.Frame.__init__(self, None, -1, _('About %s ...') % about_what, size=(400,298))
        self.Bind(wx.EVT_CLOSE, self.OnHide)

        panel = wx.Panel(self, -1, pos=(0,0), size=(0,0), )
        panel.Bind(wx.EVT_KEY_DOWN, self.OnKeyDown)
        panel.SetFocus()

        about_wximage = wx.Image(about_image, wx.BITMAP_TYPE_PNG, )
        about_wximage.Rescale(400, int(float(400)/about_wximage.Width*about_wximage.Height))
        about_wxbitmap = about_wximage.ConvertToBitmap()

        _logo_bitmap = wx.StaticBitmap(self, wx.ID_ANY, about_wxbitmap, (0, 0))
        self.bitmap = _logo_bitmap

        if "wxMSW" in wx.PlatformInfo:
            icon_size = '16x16'
        elif "wxGTK" in wx.PlatformInfo:
            icon_size = '22x22'
        elif "wxMAC" in wx.PlatformInfo:
            icon_size = '128x128'

        if icon_name is None:
            icon_name = '{appname}_winicon.png'.format(appname=_PyHocaGUI.appname)

        icon_name = os.path.expanduser(icon_name)
        if not icon_name.lower().endswith('.png'):
            icon_name = '%s.png' % icon_name

        icon_file = icon_name
        if not (os.path.isfile(str(icon_file)) or os.path.islink(str(icon_file))):
            icon_folder = 'PyHoca'
            if os.path.isdir(os.path.join(basepath.icons_basepath, _PyHocaGUI.appname)):
                icon_folder = _PyHocaGUI.appname
            icon_file = '%s/%s/%s/%s' % (basepath.icons_basepath, icon_folder, icon_size, icon_name)

        img = wx.Image(icon_file)
        if wx_major < 4:
            icon = wx.IconFromBitmap(img.ConvertToBitmap())
        else:
            icon = wx.Icon(img.ConvertToBitmap())
        self.icon = self.SetIcon(icon)

        self.CenterOnScreen()

    def OnHide(self, evt):
        """\
        Hide the About window (never close it as it is the main window of the application.

        """
        self.Show(False)

    def OnKeyDown(self, evt):
        """\
        Handle keyboard requests, so that pressing ESC can hide the About window.

        """
        keycode = evt.GetKeyCode()
        if keycode == wx.WXK_ESCAPE:
            self.Show(False)
        evt.Skip()




# -*- coding: utf-8 -*-
# vim:fenc=utf-8

# Copyright (C) 2010-2020 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
# Copyright (C) 2010-2016 by Dick Kniep <dick.kniep@lindix.nl>
#
# PyHoca GUI is free software; you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# PyHoca GUI is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program; if not, write to the
# Free Software Foundation, Inc.,
# 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.

import os
import re

import wx

import x2go.log as log
import x2go.utils as utils
from x2go import X2GOCLIENT_OS as _X2GOCLIENT_OS
from x2go import CURRENT_LOCAL_USER as _CURRENT_LOCAL_USER
from x2go import LOCAL_HOME as _LOCAL_HOME

if _X2GOCLIENT_OS != "Windows":
    import cups
else:
    import win32print

from . import basepath

class PyHocaGUI_PrintingPreferences(wx.Dialog):
    """\
    The print preferences dialog box allowing the configuration and re-configuration
    of the processing of incoming / client-side print jobs.

    """
    def __init__(self, _PyHocaGUI, profile_name=None, session_name=None, mode='edit'):
        """\
        Printing preferences dialog box (constructor).

        @param _PyHocaGUI: the master/parent object of the application
        @type _PyHocaGUI: C{obj}
        @param profile_name: session profile name
        @type profile_name: C{str}
        @param session_name: the X2Go session name of the Window that we intend to modify the name of
        @type session_name C{str}
        @param sesion_name: is this instance launched on an incoming pring job? Or are wie in editor mode.

        """
        self._PyHocaGUI = _PyHocaGUI
        self._pyhoca_logger = self._PyHocaGUI._pyhoca_logger
        self.mode = mode

        _icons_location = basepath.icons_basepath

        self._pyhoca_logger('opening client printing configuration dialog, mode: ,,%s\'\'' % self.mode, loglevel=log.loglevel_INFO)

        if self.mode == 'edit':
            _title = _('%s - Printing Preferences') % self._PyHocaGUI.appname
        else:
            _title = _('%s - Incoming Print Job from  %s (%s)') % (self._PyHocaGUI.appname, profile_name, session_name)
        wx.Dialog.__init__(self, None, -1, title=_title, style=wx.DEFAULT_DIALOG_STYLE, )
        self._PyHocaGUI._sub_windows.append(self)

        self._availablePrintActions = {
            'DIALOG': _('Open this dialog window'),
            'PDFVIEW': _('Open with PDF viewer'),
            'PDFSAVE': _('Save to a local folder'),
            'PRINT': _('Print to a local printer'),
            'PRINTCMD': _('Run custom print command'),
            }
        if self.mode != 'edit':
            self._availablePrintActions['DIALOG'] = _('<Select a print action here>')

        self._availablePrinters = {}
        if _X2GOCLIENT_OS != "Windows":
            # initialize CUPS config API and retrieve a list of local print queues
            try:
                cups.setUser(_CURRENT_LOCAL_USER)
                cups_connection = cups.Connection()
                cups_printers = cups_connection.getPrinters()
                for p in list(cups_printers.keys()):
                    self._availablePrinters.update({ p: '%s (%s)' % (p, cups_printers[p]['printer-info']), })
                self._defaultPrinter = cups_connection.getDefault()
            except RuntimeError:
                self._defaultPrinter = '#PRINTSYSTEM_UNAVAILABLE#'
        else:
            # initialize win32print API and retrieve a list of local print queues
            try:
                win32_printers = win32print.EnumPrinters(win32print.PRINTER_ENUM_LOCAL)
                for p in win32_printers:
                    if p[3]:
                        self._availablePrinters.update({p[2]: '%s (%s)' % (p[2], p[3]), })
                    else:
                        self._availablePrinters.update({p[2]: '%s' % p[2], })
                        self._defaultPrinter = win32print.GetDefaultPrinter()
            except:
                # FIXME: find out what exception could occur here.
                self._defaultPrinter = '#PRINTSYSTEM_UNAVAILABLE#'

        self.client_printing = self._PyHocaGUI.client_printing

        ###
        ### widgets for CLIENT PRINTING
        ###
        if self.mode != 'edit':
            self.PrintActionLabel = wx.StaticText(self, -1, _("Print action")+':')
        else:
            self.PrintActionLabel = wx.StaticText(self, -1, _("Default action for incoming print jobs")+':')
        self.PrintAction = wx.ComboBox(self, -1, choices=list(self._availablePrintActions.values()), style=wx.CB_DROPDOWN|wx.CB_READONLY)

        # widgets for print action PDFVIEW
        self.PdfViewCmdLabel = wx.StaticText(self, -1, _('PDF viewer command') + ':', )
        self.PdfViewCmd = wx.TextCtrl(self, -1, '', )
        self.PdfViewCmdBrowseButton = wx.BitmapButton(self, -1,
                                                      wx.Bitmap('%s/PyHoca/16x16/system-search.png' % _icons_location, wx.BITMAP_TYPE_ANY)
                                                     )

        # widgets for print action PDFSAVE
        self.PdfSaveToFolderLabel = wx.StaticText(self, -1, _('Save PDFs to folder') + ':', )
        self.PdfSaveToFolder = wx.TextCtrl(self, -1, '', )
        self.PdfSaveToFolderBrowseButton = wx.BitmapButton(self, -1,
                                                           wx.Bitmap('%s/PyHoca/16x16/system-search.png' % _icons_location, wx.BITMAP_TYPE_ANY)
                                                          )

        # widgets for print action PRINT
        self.PrintPrinterLabel = wx.StaticText(self, -1, _('Use this printer') + ':', )
        self.PrintPrinter = wx.ComboBox(self, -1, choices=list(self._availablePrinters.values()), style=wx.CB_DROPDOWN|wx.CB_READONLY)

        # widgets for print action PRINTCMD
        self.PrintCmdLabel = wx.StaticText(self, -1, _('Custom print command') + ':', )
        self.PrintCmd = wx.TextCtrl(self, -1, '', )

        if self.mode == 'edit':
            self.OKButton = wx.Button(self, wx.ID_OK, _("Ok"))
            self.ApplyButton = wx.Button(self, -1, _("Apply"))
        else:
            self.OKButton = wx.Button(self, wx.ID_OK, _("Print"))
        self.OKButton.SetDefault()
        self.CancelButton = wx.Button(self, wx.ID_CANCEL, _("Cancel"))

        self.__set_properties()
        self.__update_fields()
        self.__do_layout()

        self.Bind(wx.EVT_COMBOBOX, self.OnPrintActionChange, self.PrintAction)

        self.Bind(wx.EVT_BUTTON, self.OnPdfViewCmdBrowseButton, self.PdfViewCmdBrowseButton)
        self.Bind(wx.EVT_BUTTON, self.OnPdfSaveToFolderBrowseButton, self.PdfSaveToFolderBrowseButton)

        self.Bind(wx.EVT_BUTTON, self.OnOKButton, self.OKButton)
        if self.mode == 'edit':
            self.Bind(wx.EVT_BUTTON, self.OnApplyButton, self.ApplyButton)
        self.Bind(wx.EVT_BUTTON, self.OnCancel, self.CancelButton)


    def __set_properties(self):
        """\
        Set field properties (before layouting).

        """
        _textfield_height = self.PdfViewCmdBrowseButton.GetBestSize().GetHeight()-2
        self.PrintActionLabel.SetMinSize((-1, 16))
        self.PrintAction.SetMinSize((-1, _textfield_height+4))
        self.PdfViewCmdBrowseButton.SetMinSize(self.PdfViewCmdBrowseButton.GetBestSize())
        self.PdfSaveToFolderBrowseButton.SetMinSize(self.PdfSaveToFolderBrowseButton.GetBestSize())
        self.OKButton.SetMinSize((-1, 30))
        if self.mode == 'edit':
            self.ApplyButton.SetMinSize((-1, 30))
        self.CancelButton.SetMinSize((-1, 30))

    def __do_layout(self):
        """\
        Arrange the frame's widget layout.

        """
        sizer_1 = wx.BoxSizer(wx.VERTICAL)
        sizer_1_1 = wx.GridBagSizer(hgap=2, vgap=5)
        sizer_1_1.Add(self.PrintActionLabel, pos=(0,0), flag=wx.ALIGN_CENTER_VERTICAL|wx.RIGHT|wx.BOTTOM, border=7, )
        sizer_1_1.Add(self.PrintAction, pos=(0,1), flag=wx.BOTTOM, border=7, )

        sizer_1_1.Add(self.PdfViewCmdLabel, pos=(1,0), flag=wx.ALIGN_CENTER_VERTICAL|wx.RIGHT, border=7, )
        sizer_1_1_1 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_1_1_1.Add(self.PdfViewCmd, proportion=1, flag=wx.EXPAND|wx.RIGHT, border=7, )
        sizer_1_1_1.Add(self.PdfViewCmdBrowseButton, )
        sizer_1_1.Add(sizer_1_1_1, pos=(1,1), flag=wx.EXPAND|wx.ALIGN_CENTER_VERTICAL, )

        sizer_1_1.Add(self.PdfSaveToFolderLabel, pos=(2,0), flag=wx.ALIGN_CENTER_VERTICAL|wx.RIGHT, border=7, )
        sizer_1_1_2 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_1_1_2.Add(self.PdfSaveToFolder, proportion=1, flag=wx.EXPAND|wx.RIGHT, border=7, )
        sizer_1_1_2.Add(self.PdfSaveToFolderBrowseButton, )
        sizer_1_1.Add(sizer_1_1_2, pos=(2,1), flag=wx.EXPAND|wx.ALIGN_CENTER_VERTICAL, )

        sizer_1_1.Add(self.PrintPrinterLabel, pos=(3,0), flag=wx.ALIGN_CENTER_VERTICAL|wx.RIGHT, border=7, )
        sizer_1_1_3 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_1_1_3.Add(self.PrintPrinter, proportion=1, flag=wx.EXPAND, )
        sizer_1_1.Add(sizer_1_1_3, pos=(3,1), flag=wx.EXPAND|wx.ALIGN_CENTER_VERTICAL, )

        sizer_1_1.Add(self.PrintCmdLabel, pos=(4,0), flag=wx.ALIGN_CENTER_VERTICAL|wx.RIGHT, border=7, )
        sizer_1_1_4 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_1_1_4.Add(self.PrintCmd, proportion=1, flag=wx.EXPAND, )
        sizer_1_1.Add(sizer_1_1_4, pos=(4,1), flag=wx.EXPAND|wx.ALIGN_CENTER_VERTICAL, )
        sizer_1.Add(sizer_1_1, flag=wx.EXPAND|wx.ALL, border=10)

        # the bottom area with OK and Cancel buttons
        sizer_B = wx.BoxSizer(wx.HORIZONTAL)
        sizer_B.Add(self.OKButton, flag=wx.ALL, border=5)
        if self.mode == 'edit':
            sizer_B.Add(self.ApplyButton, flag=wx.ALL, border=5)
        sizer_B.Add(self.CancelButton, flag=wx.ALL, border=5)

        # put it all together...
        MainSizer = wx.BoxSizer(wx.VERTICAL)
        MainSizer.Add(sizer_1, proportion=1, flag=wx.EXPAND|wx.FIXED_MINSIZE)
        MainSizer.Add(sizer_B, flag=wx.ALIGN_RIGHT)
        self.SetSizerAndFit(MainSizer)

        self.SetAutoLayout(True)
        self.Layout()
        self.CentreOnScreen()
        self.Show(True)

    def __update_fields(self):
        """\
        Update field from running application.

        """
        print_action_name = self.client_printing.get_print_action(reload=True, reinit=True, return_name=True)

        self.PdfViewCmd.SetValue(self.client_printing.get_property('pdfview_cmd'))
        self.PdfSaveToFolder.SetValue(self.client_printing.get_property('save_to_folder'))
        if self._defaultPrinter != '#PRINTSYSTEM_UNAVAILABLE#':
            if self._availablePrinters:
                try:
                    _printer_name = self._availablePrinters[self.client_printing.get_property('printer')]
                except KeyError:
                    if self._defaultPrinter != None:
                        _printer_name = self._availablePrinters[self._defaultPrinter]
                    else:
                        _printer_name = self._availablePrinters[list(self._availablePrinters.keys())[0]]
            else:
                _printer_name = _('- no printers installed -')
                self.PrintPrinter.Clear()
                self.PrintPrinter.Append(_(_printer_name))
                self.PrintPrinter.Enable(False)
        else:
            _printer_name = _('- print system is not available -')
            self.PrintPrinter.Clear()
            self.PrintPrinter.Append(_(_printer_name))
            self.PrintPrinter.Enable(False)
        self.PrintPrinter.SetValue(_printer_name)
        self.PrintCmd.SetValue(self.client_printing.get_property('print_cmd'))

        if print_action_name in self._availablePrintActions:
            self.PrintAction.SetValue(self._availablePrintActions[print_action_name])
        else:
            self.PrintAction.SetValue(print_action_name)
        self._onPrintActionChange()

        if self.mode != 'edit':
            if self._print_action == 'DIALOG':
                self.OKButton.Enable(False)

    def __update_from_screen(self):
        self.client_printing.store_print_action(self._print_action, **self._print_action_properties)

    @property
    def _print_action(self):
        return [ p for p in list(self._availablePrintActions.keys()) if self._availablePrintActions[p] == self.PrintAction.GetValue() ][0]

    @property
    def _print_action_properties(self):

        # handle missing print system
        _printer = [ p for p in list(self._availablePrinters.keys()) if self._availablePrinters[p] == self.PrintPrinter.GetValue() and self._defaultPrinter != '#PRINTSYSTEM_UNAVAILABLE#' ]
        try:
            _printer = _printer[0]
        except IndexError:
            _printer = ''

        return {
            'pdfview_cmd': self.PdfViewCmd.GetValue(),
            'save_to_folder': self.PdfSaveToFolder.GetValue(),
            'printer': _printer,
            'print_cmd': self.PrintCmd.GetValue(),
        }

    def get_print_action(self):
        """\
        Retrieve the current print action.

        """
        self.__update_from_screen()
        return self.client_printing.get_print_action(reinit=True)

    def get_print_action_properties(self):
        """\
        Retrieve action properties from all users.

        """
        return self._print_action_properties

    def _onPrintActionChange(self):
        """\
        Helper method for L{OnPrintActionChange}.

        """
        if self._print_action == 'PDFVIEW':
            self.PdfViewSelected()
        elif self._print_action == 'PDFSAVE':
            self.PdfSaveToFolderSelected()
        elif self._print_action == 'PRINT':
            self.PrintPrinterSelected()
        elif self._print_action == 'PRINTCMD':
            self.PrintCmdSelected()
        else:
            self._disable_PrintOptions()
        self.__update_from_screen()

    def OnPrintActionChange(self, evt):
        """
        Gets called whenever a print action change has been requested.

        @param evt: event
        @type evt: C{obj}

        """
        self._onPrintActionChange()

    def _disable_PrintOptions(self):
        """\
        Helper method for L{PdfViewSelected}, L{PdfSaveToFolderSelected}, L{PrintPrinterSelected}, and L{PrintCmdSelected}.

        """
        self.PdfViewCmdLabel.Enable(False)
        self.PdfViewCmd.Enable(False)
        self.PdfViewCmdBrowseButton.Enable(False)
        self.PdfSaveToFolderLabel.Enable(False)
        self.PdfSaveToFolder.Enable(False)
        self.PdfSaveToFolderBrowseButton.Enable(False)
        self.PrintPrinterLabel.Enable(False)
        self.PrintPrinter.Enable(False)
        self.PrintCmdLabel.Enable(False)
        self.PrintCmd.Enable(False)

        if self.mode != 'edit':
            if self._print_action == 'DIALOG':
                self.OKButton.Enable(False)
            else:
                self.OKButton.Enable(True)

    def PdfViewSelected(self):
        """\
        Enable/disable widgets for PDFVIEW print action.

        """
        self._disable_PrintOptions()
        self.PdfViewCmdLabel.Enable(True)
        self.PdfViewCmd.Enable(True)
        self.PdfViewCmdBrowseButton.Enable(True)

    def PdfSaveToFolderSelected(self):
        """\
        Enable/disable widgets for PDFSAVE print action.

        """
        self._disable_PrintOptions()
        self.PdfSaveToFolderLabel.Enable(True)
        self.PdfSaveToFolder.Enable(True)
        self.PdfSaveToFolderBrowseButton.Enable(True)

    def PrintPrinterSelected(self):
        """\
        Enable/disable widgets for PRINT print action.

        """
        self._disable_PrintOptions()
        if self._defaultPrinter != '#PRINTSYSTEM_UNAVAILABLE#':
            self.PrintPrinterLabel.Enable(True)
            self.PrintPrinter.Enable(True)

    def PrintCmdSelected(self):
        """\
        Enable/disable widgets for PRINTCMD print action.

        """
        self._disable_PrintOptions()
        self.PrintCmdLabel.Enable(True)
        self.PrintCmd.Enable(True)

    def OnPdfViewCmdBrowseButton(self, evt):
        """\
        Gets called if the user requests to browse for a PDF view executable.

        @param evt: event
        @type evt: C{obj}

        """
        wildcard = "All files (*.*)|*"
        dlg = wx.FileDialog(
            self, message=_("Choose PDF viewer application"), defaultDir=_LOCAL_HOME,
            defaultFile="", wildcard=wildcard, style=wx.FD_OPEN | wx.FD_CHANGE_DIR )
        # Show the dialog and retrieve the user response. If it is the OK response,
        # process the data.
        if dlg.ShowModal() == wx.ID_OK:
            # This returns a Python list of files that were selected.
            _pdfview_cmd = dlg.GetPath()
            self.PdfViewCmd.SetValue(_pdfview_cmd)

    def OnPdfSaveToFolderBrowseButton(self, evt):
        """\
        Gets called if the user requests to set the PDFSAVE folder location.

        @param evt: event
        @type evt: C{obj}

        """
        _start_dir = self.PdfSaveToFolder.GetValue()
        if not utils.is_abs_path(_start_dir):
            if not _start_dir.startswith('~'):
                _start_dir = '~/%s' % _start_dir
            _start_dir = os.path.expanduser(_start_dir)
        dlg = wx.DirDialog(
            self, message=_("Choose PDF saving location"), style=1, defaultPath=_start_dir, )
        # Show the dialog and retrieve the user response. If it is the OK response,
        # process the data.
        if dlg.ShowModal() == wx.ID_OK:
            # This returns a Python list of files that were selected.
            _save_to_folder = dlg.GetPath()
            if _X2GOCLIENT_OS != 'Windows':
                if _save_to_folder.startswith(_LOCAL_HOME):
                    _save_to_folder = re.sub('^%s\/' % _LOCAL_HOME, '', _save_to_folder)
            else:
                if os.path.normpath(_save_to_folder).lower().startswith(os.path.normpath(_LOCAL_HOME).lower()):
                    _save_to_folder = re.sub('^%s\\\\' % os.path.normpath(_LOCAL_HOME).replace('\\', '\\\\'), '', _save_to_folder, flags=re.IGNORECASE)
            self.PdfSaveToFolder.SetValue(_save_to_folder)

    def __validate(self):
        """\
        Dummy field validation method. Always returns C{True}.

        """
        validateOk = True
        return validateOk

    def _apply_changes(self):
        """\
        Apply changes in the dialog to the client printing configuration.

        """
        self.__update_from_screen()
        if self.__validate():
            if self.mode == 'edit': self.client_printing.write()
            return True
        return False

    def OnOKButton(self, evt):
        """\
        Gets called if the user presses the ,,Ok'' button.

        @param evt: event
        @type evt: C{obj}

        """
        wx.BeginBusyCursor()
        if self._apply_changes():
            try: wx.EndBusyCursor()
            except: pass
            self.Close()
            self.Destroy()
        else:
            try: wx.EndBusyCursor()
            except: pass

    def OnApplyButton(self, evt):
        """\
        Gets called if the user presses the ,,Apply'' button.

        @param evt: event
        @type evt: C{obj}

        """
        wx.BeginBusyCursor()
        self._apply_changes()
        try: wx.EndBusyCursor()
        except: pass

    def OnCancel(self, evt):
        """\
        Gets called if the user presses the ,,Cancel'' button.

        @param evt: event
        @type evt: C{obj}

        """
        self.client_printing.load()
        self.Close()
        self.Destroy()

    def Destroy(self):
        """\
        Tidy up some stuff in the main application instance before allowing desctruction of the
        printing preferences window.

        """
        try:
            self._PyHocaGUI._sub_windows.remove(self)
        except ValueError:
            pass
        wx.Dialog.Destroy(self)



# -*- coding: utf-8 -*-
# vim:fenc=utf-8

# Copyright (C) 2010-2020 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
# Copyright (C) 2010-2016 by Dick Kniep <dick.kniep@lindix.nl>
#
# PyHoca GUI is free software; you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# PyHoca GUI is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program; if not, write to the
# Free Software Foundation, Inc.,
# 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.

import sys
import wx
from x2go import X2GOCLIENT_OS
from x2go import log
if X2GOCLIENT_OS in ('Linux', 'Mac'):
    try:
        # try the GTK3 way for notification support first...
        from gi import require_version
        require_version('Notify', '0.7')
        from gi.repository import Notify as _Notify
    except ImportError:
        # GTK3 notifications are unavailable,
        # hopefully pynotify / GTK2-based notifications are available then...
        import pynotify as _Notify
from . import basepath

import x2go.utils as utils

class NotSupportedException(BaseException): pass
class PyHocaNotificationException(BaseException): pass

def notification_service_available():
    return _Notify.get_server_info()[0]

class stdout_NotifierPopup(object):
    """\
    L{PyHocaGUI} stdout notifications, used when no desktop notification service is available.

    """
    title = {}
    text = {}
    icon = {}
    timeout = {}

    def __init__(self, _PyHocaGUI):
        """\
        Notifier popup (constructor).

        @param _PyHocaGUI: main application instance
        @type _PyHocaGUI: C{obj}

        """
        self._PyHocaGUI = _PyHocaGUI
        self._pyhoca_logger = self._PyHocaGUI._pyhoca_logger

    def prepare(self, context, title=None, text=None, icon=None, timeout=None):
        """\
        Prepare a notification that gets sent to stdout later (by the L{send()} method).

        Use C{context} as a unique identifier. When sending the notification later, C{context}
        will unequivocally map to the notification content that shall get sent.

        @param context: a unique identifier for this notification preparation
        @type context: C{str}
        @param title: notification title
        @type title: C{str}
        @param text: notification text
        @type text: C{str}
        @param icon: icon name for an icon that appears with the notification
        @type icon: C{str}
        @param timeout: let notification disappear after C{<timeout>} milliseconds
        @type timeout: C{int}

        """
        if title is not None:
            self.title[context] = title
        if text is not None:
            self.text[context] = text
        if icon is not None:
            self.icon[context] = icon
        if timeout is not None:
            self.timeout[context] = timeout

    def send(self, title=None, text=None, context=None, icon=None, timeout=8000):
        """\
        Send notifications directly to stdout (or use a prepared notification).

        @param title: notification title
        @type title: C{str}
        @param text: notification text
        @type text: C{str}
        @param context: an identifier that refers to a prepared notification
        @type context: C{str}
        @param icon: icon name for an icon that appears with the notification
        @type icon: C{str}
        @param timeout: let notification disappear after C{<timeout>} milliseconds
        @type timeout: C{int}

        @raise PyHocaNotificationException: if notification failed

        """
        if context is not None:
            try:
                title = self.title[context]
                del self.title[context]
            except KeyError:
                pass
            try:
                text = self.text[context]
                del self.text[context]
            except KeyError:
                pass
            try:
                icon = self.icon[context]
                del self.icon[context]
            except KeyError:
                pass
            try:
                timeout = self.timeout[context]
                del self.timeout[context]
            except KeyError:
                pass

        if title and text:
            if sys.version_info[0] >= 3:
                self._pyhoca_logger('[%s] %s' % (title, text), loglevel=log.loglevel_NOTICE)
            else:
                self._pyhoca_logger('[%s] %s' % (title.encode(utils.get_encoding()), text.encode(utils.get_encoding())), loglevel=log.loglevel_NOTICE)

    def Close(self):
        """\
        Provide a C{Close()} method which does nothing.

        """
        pass

    def Destroy(self):
        """\
        Provide a C{Destroy()} method which does nothing.

        """
        pass


class libnotify_NotifierPopup(object):
    """\
    L{PyHocaGUI} notification utilizing C{libnotify}, used on Linux/Unix OS.

    """
    title = {}
    text = {}
    icon = {}
    timeout = {}

    def __init__(self, _PyHocaGUI):
        """\
        Notifier popup (constructor).

        @param _PyHocaGUI: main application instance
        @type _PyHocaGUI: C{obj}

        """
        self._PyHocaGUI = _PyHocaGUI
        self._pyhoca_logger = self._PyHocaGUI._pyhoca_logger

        if not _Notify.init("PyHocaGUI"):
            raise NotSupportedException

    def prepare(self, context, title=None, text=None, icon=None, timeout=None):
        """\
        Prepare a notification that gets sent to C{libnotify} later (by the L{send()} method).

        Use C{context} as a unique identifier. When sending the notification later, C{context}
        will unequivocally map to the notification content that shall get sent.

        @param context: a unique identifier for this notification preparation
        @type context: C{str}
        @param title: notification title
        @type title: C{str}
        @param text: notification text
        @type text: C{str}
        @param icon: icon name for an icon that appears with the notification
        @type icon: C{str}
        @param timeout: let notification disappear after C{<timeout>} milliseconds
        @type timeout: C{int}

        """
        if title is not None:
            self.title[context] = title
        if text is not None:
            self.text[context] = text
        if icon is not None:
            self.icon[context] = icon
        if timeout is not None:
            self.timeout[context] = timeout

    def send(self, title=None, text=None, context=None, icon=None, timeout=8000):
        """\
        Send notifications directly (or use a prepared notification).

        @param title: notification title
        @type title: C{str}
        @param text: notification text
        @type text: C{str}
        @param context: an identifier that refers to a prepared notification
        @type context: C{str}
        @param icon: icon name for an icon that appears with the notification
        @type icon: C{str}
        @param timeout: let notification disappear after C{<timeout>} milliseconds
        @type timeout: C{int}

        @raise PyHocaNotificationException: if notification failed

        """
        if context is not None:
            try:
                title = self.title[context]
                del self.title[context]
            except KeyError:
                pass
            try:
                text = self.text[context]
                del self.text[context]
            except KeyError:
                pass
            try:
                icon = self.icon[context]
                del self.icon[context]
            except KeyError:
                pass
            try:
                timeout = self.timeout[context]
                del self.timeout[context]
            except KeyError:
                pass

        _icons_location = basepath.icons_basepath
        if icon:
            icon = 'file://%s/PyHoca/32x32/%s.png' % (_icons_location, icon)

        if sys.version_info[0] >= 3:
            self._pyhoca_logger('[%s] %s' % (title, text), loglevel=log.loglevel_NOTICE)
        else:
            self._pyhoca_logger('[%s] %s' % (title.encode(utils.get_encoding()), text.encode(utils.get_encoding())), loglevel=log.loglevel_NOTICE)

        try:
            if not self._PyHocaGUI.disable_notifications and title and text:
                try:
                    n = _Notify.Notification.new(title, text, icon)
                    n.set_urgency(_Notify.Urgency.NORMAL)
                except AttributeError:
                    # handle pynotify / GTK2-based notifications
                    n = _Notify.Notification(title, text, icon)
                    n.set_urgency(_Notify.URGENCY_NORMAL)
                n.set_timeout(timeout)
                n.show()
        except:
            pass

    def Close(self):
        """\
        Provide a C{Close()} method which does nothing.

        """
        pass

    def Destroy(self):
        """\
        Provide a C{Destroy()} method which does nothing.

        """
        pass


class showballoon_NotifierPopup(object):
    """\
    L{PyHocaGUI} notification utilizing C{wx.TaskBarIcon.ShowBalloon()}, used on Windows OS.

    """
    title = {}
    text = {}
    icon = {}
    timeout = {}

    def __init__(self, _about):
        """\
        Notifier popup (constructor).

        @param _about: main application window
        @type _about: C{obj}

        """
        self._PyHocaGUI = _about._PyHocaGUI
        self._pyhoca_logger = self._PyHocaGUI._pyhoca_logger

    def prepare(self, context, title=None, text=None, icon=None, timeout=None):
        """\
        Prepare a notification that gets sent to C{libnotify} later (by the L{send()} method).

        Use C{context} as a unique identifier. When sending the notification later, C{context}
        will unequivocally map to the notification content that shall get sent.

        @param context: a unique identifier for this notification preparation
        @type context: C{str}
        @param title: notification title
        @type title: C{str}
        @param text: notification text
        @type text: C{str}
        @param icon: icon name for an icon that appears with the notification
        @type icon: C{str}
        @param timeout: let notification disappear after C{<timeout>} milliseconds
        @type timeout: C{int}

        """
        if title is not None:
            self.title[context] = title
        if text is not None:
            self.text[context] = text
        if icon is not None:
            self.icon[context] = icon
        if timeout is not None:
            self.timeout[context] = timeout

    def send(self, title=None, text=None, context=None, icon=None, timeout=8000):
        """\
        Send notifications directly (or use a prepared notification).

        @param title: notification title
        @type title: C{str}
        @param text: notification text
        @type text: C{str}
        @param context: an identifier that refers to a prepared notification
        @type context: C{str}
        @param icon: icon name for an icon that appears with the notification
        @type icon: C{str}
        @param timeout: let notification disappear after C{<timeout>} milliseconds
        @type timeout: C{int}

        """
        if context is not None:
            try:
                title = self.title[context]
                del self.title[context]
            except KeyError:
                pass
            try:
                text = self.text[context]
                del self.text[context]
            except KeyError:
                pass
            try:
                icon = self.icon[context]
                del self.icon[context]
            except KeyError:
                pass
            try:
                timeout = self.timeout[context]
                del self.timeout[context]
            except KeyError:
                pass

        # libnotify timeouts are given in millisecs, on Windows we use seconds...
        timeout = timeout / 1000

        _icon_map_wx = {
            'audio_error': wx.ICON_ERROR,
            'auth_success': wx.ICON_INFORMATION,
            'auth_failed': wx.ICON_WARNING,
            'auth_error': wx.ICON_ERROR,
            'auth_disconnect': wx.ICON_INFORMATION,
            'profile_add': wx.ICON_INFORMATION,
            'profile_delete': wx.ICON_INFORMATION,
            'profile_edit': wx.ICON_INFORMATION,
            'profile_save': wx.ICON_INFORMATION,
            'profile_error': wx.ICON_ERROR,
            'session_cleanall': wx.ICON_INFORMATION,
            'session_error': wx.ICON_ERROR,
            'session_pause': wx.ICON_INFORMATION,
            'session_printing': wx.ICON_INFORMATION,
            'session_resume': wx.ICON_INFORMATION,
            'session_start': wx.ICON_INFORMATION,
            'session_terminate': wx.ICON_INFORMATION,
            'session_warning': wx.ICON_WARNING,
        }
        if icon in list(_icon_map_wx.keys()):
           icon = _icon_map_wx[icon]
        else:
           icon = wx.ICON_INFORMATION

        try:
            if not self._PyHocaGUI.disable_notifications and title and text:
                # you will need wxPython >= 2.9 for this
                self._PyHocaGUI.taskbar.ShowBalloon(
                    title,
                    text,
                    timeout*1000,
                    icon,
                )
        except:
            pass

        if title and text:
            # on Windows some error messages are already encoded, some are not, depending from which module they come
            try: _title = title.encode(utils.get_encoding())
            except: _title = title
            try: _text = text.encode(utils.get_encoding())
            except: _text = text

            try: self._pyhoca_logger('['+_title+'] '+_text, loglevel=log.loglevel_NOTICE)
            except UnicodeDecodeError: self._pyhoca_logger('Unicode error occurred while rendering a log message...', loglevel=log.loglevel_WARN)

    def Close(self):
        """\
        Provide a C{Close()} method which does nothing.

        """
        pass

    def Destroy(self):
        """\
        Provide a C{Destroy()} method which does nothing.

        """
        pass


class notificationmessage_NotifierPopup(object):
    """\
    L{PyHocaGUI} notification utilizing C{wx.NotificationMessage()}, used on Windows OS.

    Note: C{wx.NotificationMessage()} has only been added to wxPython in version 2.9.x.

    """
    title = {}
    text = {}
    icon = {}
    timeout = {}

    def __init__(self, _about):
        """\
        Notifier popup (constructor).

        @param _about: main application window
        @type _about: C{obj}

        """
        self._PyHocaGUI = _about._PyHocaGUI
        self._pyhoca_logger = self._PyHocaGUI._pyhoca_logger

    def prepare(self, context, title=None, text=None, icon=None, timeout=None):
        """\
        Prepare a notification that gets sent to C{libnotify} later (by the L{send()} method).

        Use C{context} as a unique identifier. When sending the notification later, C{context}
        will unequivocally map to the notification content that shall get sent.

        @param context: a unique identifier for this notification preparation
        @type context: C{str}
        @param title: notification title
        @type title: C{str}
        @param text: notification text
        @type text: C{str}
        @param icon: icon name for an icon that appears with the notification
        @type icon: C{str}
        @param timeout: let notification disappear after C{<timeout>} milliseconds
        @type timeout: C{int}

        """
        if title is not None:
            self.title[context] = title
        if text is not None:
            self.text[context] = text
        if icon is not None:
            self.icon[context] = icon
        if timeout is not None:
            self.timeout[context] = timeout

    def send(self, title=None, text=None, context=None, icon=None, timeout=8000):
        """\
        Send notifications directly (or use a prepared notification).

        @param title: notification title
        @type title: C{str}
        @param text: notification text
        @type text: C{str}
        @param context: an identifier that refers to a prepared notification
        @type context: C{str}
        @param icon: icon name for an icon that appears with the notification
        @type icon: C{str}
        @param timeout: let notification disappear after C{<timeout>} milliseconds
        @type timeout: C{int}

        """
        if context is not None:
            try:
                title = self.title[context]
                del self.title[context]
            except KeyError:
                pass
            try:
                text = self.text[context]
                del self.text[context]
            except KeyError:
                pass
            try:
                icon = self.icon[context]
                del self.icon[context]
            except KeyError:
                pass
            try:
                timeout = self.timeout[context]
                del self.timeout[context]
            except KeyError:
                pass

        # libnotify timeouts are given in millisecs, on Windows we use seconds...
        timeout = timeout / 1000

        _icon_map_wx = {
            'audio_error': wx.ICON_ERROR,
            'auth_success': wx.ICON_INFORMATION,
            'auth_failed': wx.ICON_WARNING,
            'auth_error': wx.ICON_ERROR,
            'auth_disconnect': wx.ICON_INFORMATION,
            'profile_add': wx.ICON_INFORMATION,
            'profile_delete': wx.ICON_INFORMATION,
            'profile_edit': wx.ICON_INFORMATION,
            'profile_save': wx.ICON_INFORMATION,
            'profile_error': wx.ICON_ERROR,
            'session_cleanall': wx.ICON_INFORMATION,
            'session_error': wx.ICON_ERROR,
            'session_pause': wx.ICON_INFORMATION,
            'session_printing': wx.ICON_INFORMATION,
            'session_resume': wx.ICON_INFORMATION,
            'session_start': wx.ICON_INFORMATION,
            'session_terminate': wx.ICON_INFORMATION,
            'session_warning': wx.ICON_WARNING,
        }
        if icon in list(_icon_map_wx.keys()):
           icon = _icon_map_wx[icon]
        else:
           icon = wx.ICON_INFORMATION

        try:
            if not self._PyHocaGUI.disable_notifications and title and text:
                # you will need wxPython >= 2.9 for this
                _notification_msg = wx.NotificationMessage()
                _notification_msg.SetTitle(title)
                _notification_msg.SetMessage(text)
                _notification_msg.SetParent(self._PyHocaGUI.about)
                _notification_msg.SetFlags(icon)
                _notification_msg.Show(timeout=timeout)
        except:
            # if we are running wxPython 2.8, we ignore missing
            # wx.NotificationMessage class
            pass

        if title and text:
            # on Windows some error messages are already encoded, some are not, depending from which module they come
            try: _title = title.encode(utils.get_encoding())
            except: _title = title
            try: _text = text.encode(utils.get_encoding())
            except: _text = text

            try: self._pyhoca_logger('['+_title+'] '+_text, loglevel=log.loglevel_NOTICE)
            except UnicodeDecodeError: self._pyhoca_logger('Unicode error occurred while rendering a log message...', loglevel=log.loglevel_WARN)

    def Close(self):
        """\
        Provide a C{Close()} method which does nothing.

        """
        pass

    def Destroy(self):
        """\
        Provide a C{Destroy()} method which does nothing.

        """
        pass



# -*- coding: utf-8 -*-
# vim:fenc=utf-8

# Copyright (C) 2010-2020 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
# Copyright (C) 2010-2016 by Dick Kniep <dick.kniep@lindix.nl>
#
# PyHoca GUI is free software; you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# PyHoca GUI is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program; if not, write to the
# Free Software Foundation, Inc.,
# 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.

import os.path
import wx
import io
import base64

import x2go.defaults
import x2go.x2go_exceptions

from . import basepath

wx_major = int(wx.__version__.split('.')[0])

def _legacy_wrap_AppendMenu(instance, *args, **kwargs):
    if wx_major < 4:
        return wx.Menu.AppendMenu(instance, *args, **kwargs)
    else:
        return wx.Menu.Append(instance, *args, **kwargs)

class PyHocaGUI_Menu_TaskbarManageProfile(wx.Menu):
    """\
    Individual profile management submenu: copy, use as template or delete session profile.

    """
    def __init__(self, _PyHocaGUI, caller=None, profile_name=None):
        """\
        Individual profile management submenu (constructor).

        @param _PyHocaGUI: main application instance
        @type _PyHocaGUI: C{obj}
        @param caller: unused
        @type caller: C{None}
        @param profile_name: session profile name this submenu is for
        @type profile_name: C{str}

        """
        self._PyHocaGUI = _PyHocaGUI
        self._pyhoca_logger = self._PyHocaGUI._pyhoca_logger

        wx.Menu.__init__(self)

        ID_EDITPROFILE = wx.NewId()
        ID_COPYPROFILE = wx.NewId()
        ID_EXPORTPROFILE = wx.NewId()
        ID_DELETEPROFILE = wx.NewId()

        # preparing profile_name information for the main PyHocaGUI instance
        self._PyHocaGUI._eventid_profilenames_map[ID_EDITPROFILE] = \
            self._PyHocaGUI._eventid_profilenames_map[ID_COPYPROFILE] = \
            self._PyHocaGUI._eventid_profilenames_map[ID_EXPORTPROFILE] = \
            self._PyHocaGUI._eventid_profilenames_map[ID_DELETEPROFILE] = profile_name

        if self._PyHocaGUI.session_profiles.is_mutable(profile_name):
            self.Append(ID_EDITPROFILE, _("Edit Profile"))
        else:
            self.Append(ID_EDITPROFILE, _("View Profile"))
        self._PyHocaGUI.Bind(wx.EVT_MENU, self._PyHocaGUI.OnProfileEdit, id=ID_EDITPROFILE)

        if not self._PyHocaGUI.args.single_session_profile:

            self.AppendSeparator()

            if self._PyHocaGUI.session_profiles.is_mutable(profile_name):
                self.Append(ID_COPYPROFILE, _("Use as Template for New Profile"))
                self._PyHocaGUI.Bind(wx.EVT_MENU, self._PyHocaGUI.OnProfileCopy, id=ID_COPYPROFILE)
                self.AppendSeparator()

            self.Append(ID_EXPORTPROFILE, _("Export Profile"))
            self._PyHocaGUI.Bind(wx.EVT_MENU, self._PyHocaGUI.OnProfileExport, id=ID_EXPORTPROFILE)

            if self._PyHocaGUI.session_profiles.is_mutable(profile_name):
                self.Append(ID_DELETEPROFILE, _("Delete Profile"))
                self._PyHocaGUI.Bind(wx.EVT_MENU, self._PyHocaGUI.OnProfileDelete, id=ID_DELETEPROFILE)

    def AppendMenu(self, *args, **kwargs):
        return _legacy_wrap_AppendMenu(self, *args, **kwargs)


class PyHocaGUI_Menu_TaskbarOptionsManager(wx.Menu):
    """\
    Right-click menu of the L{PyHocaGUI} systray icon.

    """
    def __init__(self, _PyHocaGUI, caller=None):
        """\
        Client and profile management menu of L{PyHocaGUI} (constructor).

        @param _PyHocaGUI: main application instance
        @type _PyHocaGUI: C{obj}
        @param caller: unused
        @type caller: C{None}

        """
        self._PyHocaGUI = _PyHocaGUI
        self._pyhoca_logger = self._PyHocaGUI._pyhoca_logger

        wx.Menu.__init__(self)

        ID_ABOUT = wx.NewId()
        ID_ABOUT_PYTHONX2GO = wx.NewId()
        self.Append(ID_ABOUT, _("About %s (%s)...") % (self._PyHocaGUI.appname, self._PyHocaGUI.version))
        self.Append(ID_ABOUT_PYTHONX2GO, _("About %s (%s)...") % ("Python X2Go", x2go.__VERSION__))
        self._PyHocaGUI.Bind(wx.EVT_MENU, self._PyHocaGUI.OnAbout, id=ID_ABOUT)
        self._PyHocaGUI.Bind(wx.EVT_MENU, self._PyHocaGUI.OnAboutPythonX2Go, id=ID_ABOUT_PYTHONX2GO)

        if not self._PyHocaGUI.restricted_trayicon:

            self.AppendSeparator()

            if not self._PyHocaGUI.args.single_session_profile:

                ID_PROFILEMANAGER = wx.NewId()
                _maintain_profiles_item = self.AppendMenu(ID_PROFILEMANAGER,
                                                          _("Profile Manager"),
                                                          PyHocaGUI_Menu_TaskbarProfileNames(self._PyHocaGUI,
                                                                                             caller=self,
                                                                                             filter_profiles=[],
                                                                                             disabled_profiles=self._PyHocaGUI.client_connected_profiles(return_profile_names=True) + self._PyHocaGUI._temp_disabled_profile_names,
                                                                                             submenu=PyHocaGUI_Menu_TaskbarManageProfile,
                                                                                             group_menus=True,
                                                                                            )
                                                         )
                if self._PyHocaGUI.profilemanager_disabled:
                    _maintain_profiles_item.Enable(False)

                self.AppendSeparator()

            elif self._PyHocaGUI.session_profiles.has_profile_name(self._PyHocaGUI.args.session_profile):
                ID_SINGLEPROFILEMANAGER = wx.NewId()
                _maintain_profile_item = self.AppendMenu(ID_SINGLEPROFILEMANAGER,
                                                         _('Manage Session Profile'),
                                                         PyHocaGUI_Menu_TaskbarManageProfile(self._PyHocaGUI, caller=self, profile_name=self._PyHocaGUI.args.session_profile),
                                                        )
                if self._PyHocaGUI.args.session_profile in self._PyHocaGUI.client_connected_profiles(return_profile_names=True):
                    _maintain_profile_item.Enable(False)
                self.AppendSeparator()

        if self._PyHocaGUI.with_brokerage and self._PyHocaGUI.session_profiles.is_broker_authenticated():
            ID_BROKER_DISCONNECT = wx.NewId()
            self.Append(ID_BROKER_DISCONNECT, _("Disconnect from session broker"))
            self._PyHocaGUI.Bind(wx.EVT_MENU, self._PyHocaGUI.OnBrokerDisconnect, id=ID_BROKER_DISCONNECT)
            self.AppendSeparator()

        ID_PRINTINGPREFS = wx.NewId()
        _printingprefs_item = self.Append(ID_PRINTINGPREFS, _("Printing Preferences"))
        self._PyHocaGUI.Bind(wx.EVT_MENU, self._PyHocaGUI.OnPrintingPreferences, id=ID_PRINTINGPREFS)
        if self._PyHocaGUI.printingprefs_disabled:
            _printingprefs_item.Enable(False)

        if not self._PyHocaGUI.restricted_trayicon:

            ID_OPTIONS = wx.NewId()
            _options_item = self.Append(ID_OPTIONS, _("Client Options"))
            self._PyHocaGUI.Bind(wx.EVT_MENU, self._PyHocaGUI.OnOptions, id=ID_OPTIONS)
            if self._PyHocaGUI.options_disabled:
                _options_item.Enable(False)

        self.AppendSeparator()

        ID_EXIT = wx.NewId()
        self.Append(ID_EXIT, _("E&xit"))
        self._PyHocaGUI.Bind(wx.EVT_MENU, self._PyHocaGUI.OnTaskbarExit, id=ID_EXIT)

    def AppendMenu(self, *args, **kwargs):
        return _legacy_wrap_AppendMenu(self, *args, **kwargs)


class PyHocaGUI_Menu_TaskbarSessionActions(wx.Menu):
    """\
    Session action submenu for individual sessions: start, resume, suspend, terminate etc. sessions.

    """
    def __init__(self, _PyHocaGUI, caller=None, profile_name=None, session_name=None, session_info=None, status=None):
        """\
        Session action submenu (constructor).

        @param _PyHocaGUI: main application instance
        @type _PyHocaGUI: C{obj}
        @param caller: unused
        @type caller: C{None}
        @param profile_name: session profile name this submenu is for
        @type profile_name: C{str}
        @param session_name: X2Go session name this submenu is for
        @type session_name: C{str}
        @param session_info: session info object (C{X2GoServerSessionInfo*} from Python X2Go)
        @type session_info: C{obj}
        @param status: status of this session (R for running, S for suspended)
        @type status: C{str}

        """
        self._PyHocaGUI = _PyHocaGUI
        self._pyhoca_logger = self._PyHocaGUI._pyhoca_logger

        wx.Menu.__init__(self)

        ID_RAISESESSION = wx.NewId()
        ID_RENAMESESSION = wx.NewId()
        ID_TRANSFERSESSION = wx.NewId()
        ID_TRANSFERSESSION_DISABLED = wx.NewId()
        ID_RESUMESESSION = wx.NewId()
        ID_RESUMESESSION_DISABLED = wx.NewId()
        ID_REFRESHMENU = wx.NewId()
        ID_SUSPENDSESSION = wx.NewId()
        ID_TERMINATESESSION = wx.NewId()

        # preparing profile_name information for the main PyHocaGUI instance
        self._PyHocaGUI._eventid_profilenames_map[ID_RAISESESSION] = \
            self._PyHocaGUI._eventid_profilenames_map[ID_RENAMESESSION] = \
            self._PyHocaGUI._eventid_profilenames_map[ID_TRANSFERSESSION] = \
            self._PyHocaGUI._eventid_profilenames_map[ID_RESUMESESSION] = \
            self._PyHocaGUI._eventid_profilenames_map[ID_REFRESHMENU] = \
            self._PyHocaGUI._eventid_profilenames_map[ID_SUSPENDSESSION] = \
            self._PyHocaGUI._eventid_profilenames_map[ID_TERMINATESESSION] = profile_name

        # preparing session_name information for the main PyHocaGUI instance
        self._PyHocaGUI._eventid_sessionnames_map[ID_RAISESESSION] = \
            self._PyHocaGUI._eventid_sessionnames_map[ID_RENAMESESSION] = \
            self._PyHocaGUI._eventid_sessionnames_map[ID_TRANSFERSESSION] = \
            self._PyHocaGUI._eventid_sessionnames_map[ID_RESUMESESSION] = \
            self._PyHocaGUI._eventid_sessionnames_map[ID_REFRESHMENU] = \
            self._PyHocaGUI._eventid_sessionnames_map[ID_SUSPENDSESSION] = \
            self._PyHocaGUI._eventid_sessionnames_map[ID_TERMINATESESSION] = session_name

        _s = self._PyHocaGUI.get_session_of_session_name(session_name, return_object=True, match_profile_name=profile_name)
        _session_status = status
        if session_info is not None:
            _session_status = session_info.get_status()
        elif _s.get_session_info() is not None:
            _session_status = _s.get_session_info().get_status()

        if _s is not None and \
           _s.get_session_type() in ('D', 'S') and \
           _session_status == 'R' and \
           not _s.is_published_applications_provider():

            self.Append(wx.NewId(), _("Window title") + ": " + _s.get_session_title())
            self.AppendSeparator()

        if _session_status == 'S':

            if _s is not None and _s.is_color_depth_ok():
                _rs = self.Append(ID_RESUMESESSION, _("Resume Session"))
            else:
                _rs = self.Append(ID_RESUMESESSION_DISABLED, _("Resume Session (not possible)"))
                _rs.Enable(False)

            if session_info is not None and session_info.is_published_applications_provider() and not self._PyHocaGUI.get_profile_config(profile_name, 'published'):
                _rs.Enable(False)

        elif _session_status == 'R':

            if not session_name in self._PyHocaGUI.client_associated_sessions_of_profile_name(profile_name, return_session_names=True):

                if _s is not None and _s.is_color_depth_ok():
                    self.Append(ID_TRANSFERSESSION, _("Transfer Session"))
                else:
                    _ts = self.Append(ID_TRANSFERSESSION_DISABLED, _("Transfer Session (not possible)"))
                    _ts.Enable(False)

            if not _s.is_shadow_session():
                if self._PyHocaGUI.disconnect_on_suspend and self._PyHocaGUI.exit_on_disconnect and _s.has_terminal_session():
                    _ss = self.Append(ID_SUSPENDSESSION, _("Suspend Session (and disconnect/exit)"))
                elif self._PyHocaGUI.disconnect_on_suspend and _s.has_terminal_session():
                    _ss = self.Append(ID_SUSPENDSESSION, _("Suspend Session (and disconnect)"))
                else:
                    _ss = self.Append(ID_SUSPENDSESSION, _("Suspend Session"))

                if _s.is_published_applications_provider() and not self._PyHocaGUI.get_profile_config(profile_name, 'published'):
                    _ss.Enable(False)
        if not _s.is_shadow_session():
            if self._PyHocaGUI.disconnect_on_terminate and self._PyHocaGUI.exit_on_disconnect and _s.has_terminal_session():
                self.Append(ID_SUSPENDSESSION, _("Terminate Session (and disconnect/exit)"))
            elif self._PyHocaGUI.disconnect_on_terminate and _s.has_terminal_session():
                self.Append(ID_TERMINATESESSION, _("Terminate Session (and disconnect)"))
            else:
                self.Append(ID_TERMINATESESSION, _("Terminate Session"))
        else:
            if self._PyHocaGUI.disconnect_on_terminate and self._PyHocaGUI.exit_on_disconnect and _s.has_terminal_session():
                self.Append(ID_SUSPENDSESSION, _("End Desktop Sharing (and disconnect/exit)"))
            elif self._PyHocaGUI.disconnect_on_terminate and _s.has_terminal_session():
                self.Append(ID_TERMINATESESSION, _("End Desktop Sharing (and disconnect)"))
            else:
                self.Append(ID_TERMINATESESSION, _("End Desktop Sharing"))

        if _s is not None and _s.is_published_applications_provider() and self._PyHocaGUI.get_profile_config(profile_name, 'published'):
            self.AppendSeparator()
            self.Append(ID_REFRESHMENU, _("Refresh menu tree"))

        if _s is not None and \
           _s.get_session_type() in ('D', 'S') and \
           not _s.is_published_applications_provider() and \
           _session_status == 'R' and \
           _s in self._PyHocaGUI.client_associated_sessions_of_profile_name(profile_name, return_objects=True):

            self.AppendSeparator()
            self.Append(ID_RENAMESESSION, _("Rename Session Window"))
            self.Append(ID_RAISESESSION, _("Show Session Window"))

        self._PyHocaGUI.Bind(wx.EVT_MENU, self._PyHocaGUI.OnSessionFocus, id=ID_RAISESESSION)
        self._PyHocaGUI.Bind(wx.EVT_MENU, self._PyHocaGUI.OnSessionRename, id=ID_RENAMESESSION)
        self._PyHocaGUI.Bind(wx.EVT_MENU, self._PyHocaGUI.OnSessionResume, id=ID_RESUMESESSION)
        self._PyHocaGUI.Bind(wx.EVT_MENU, self._PyHocaGUI.OnSessionResume, id=ID_TRANSFERSESSION)
        self._PyHocaGUI.Bind(wx.EVT_MENU, self._PyHocaGUI.OnPubAppRefreshMenu, id=ID_REFRESHMENU)
        self._PyHocaGUI.Bind(wx.EVT_MENU, self._PyHocaGUI.OnSessionSuspend, id=ID_SUSPENDSESSION)
        self._PyHocaGUI.Bind(wx.EVT_MENU, self._PyHocaGUI.OnSessionTerminate, id=ID_TERMINATESESSION)

    def AppendMenu(self, *args, **kwargs):
        return _legacy_wrap_AppendMenu(self, *args, **kwargs)


class PyHocaGUI_Menu_TaskbarProfileSharedFolders(wx.Menu):
    """\
    Submenu that manages folder sharing per connected session profile.

    """
    def __init__(self, _PyHocaGUI, caller=None, profile_name=None):
        """\
        Folder sharing submenu (constructor).

        @param _PyHocaGUI: main application instance
        @type _PyHocaGUI: C{obj}
        @param caller: unused
        @type caller: C{None}
        @param profile_name: session profile name this submenu is for
        @type profile_name: C{str}

        """
        self._PyHocaGUI = _PyHocaGUI
        self._pyhoca_logger = self._PyHocaGUI._pyhoca_logger

        wx.Menu.__init__(self)

        ID_SHARECUSTOMLOCALFOLDER = wx.NewId()
        ID_UNSHAREALLLOCALFOLDERS = wx.NewId()
        ID_REMEMBERSHAREDFOLDERS = wx.NewId()

        # preparing profile_name information for the main PyHocaGUI instance
        self._PyHocaGUI._eventid_profilenames_map[ID_SHARECUSTOMLOCALFOLDER] = \
            self._PyHocaGUI._eventid_profilenames_map[ID_UNSHAREALLLOCALFOLDERS] = \
            self._PyHocaGUI._eventid_profilenames_map[ID_REMEMBERSHAREDFOLDERS] = profile_name

        self.Append(ID_SHARECUSTOMLOCALFOLDER, _("&Share custom local folder"))

        self.AppendSeparator()
        self._PyHocaGUI._eventid_unshared_folders_map={}

        _exported_folders = self._PyHocaGUI.get_profile_config(profile_name, 'export')
        _shared_folders = self._PyHocaGUI._X2GoClient__profile_get_shared_folders(profile_name=profile_name, check_list_mounts=True) or []
        _sharable_folders = list(_exported_folders.keys())
        _unshared_folders = [ f for f in _sharable_folders if f and f not in _shared_folders ]

        self._PyHocaGUI._eventid_unshared_folders_map = {}
        if _unshared_folders:
            self.Append(wx.NewId(), _('Share:'))
            for _unshared_folder in _unshared_folders:
                ID_THISFOLDER = wx.NewId()
                self.Append(ID_THISFOLDER, "      %s" % _unshared_folder)
                self._PyHocaGUI._eventid_profilenames_map[ID_THISFOLDER] = profile_name
                self._PyHocaGUI._eventid_unshared_folders_map[ID_THISFOLDER] = _unshared_folder
                self._PyHocaGUI.Bind(wx.EVT_MENU, self._PyHocaGUI.OnShareLocalFolder, id=ID_THISFOLDER)

        self._PyHocaGUI._eventid_shared_folders_map = {}
        if _shared_folders:
            self.Append(wx.NewId(), _('Unshare:'))
            for _shared_folder in _shared_folders:
                ID_THISFOLDER = wx.NewId()
                self.Append(ID_THISFOLDER, "      %s" % _shared_folder)
                self._PyHocaGUI._eventid_profilenames_map[ID_THISFOLDER] = profile_name
                self._PyHocaGUI._eventid_shared_folders_map[ID_THISFOLDER] = _shared_folder
                self._PyHocaGUI.Bind(wx.EVT_MENU, self._PyHocaGUI.OnUnshareLocalFolder, id=ID_THISFOLDER)

        _unshare_folders = self.Append(ID_UNSHAREALLLOCALFOLDERS, _("Unshare &all local folders"))
        if not _shared_folders:
            _unshare_folders.Enable(False)

        self.AppendSeparator()

        _remember_shared_folders_item = self.AppendCheckItem(ID_REMEMBERSHAREDFOLDERS, _("Restore shares in next session"))
        if profile_name not in self._PyHocaGUI._remember_shared_folders:
            self._PyHocaGUI._remember_shared_folders[profile_name] = self._PyHocaGUI.get_profile_config(profile_name, 'restoreexports')
        _remember_shared_folders_item.Check(self._PyHocaGUI._remember_shared_folders[profile_name])

        self._PyHocaGUI.Bind(wx.EVT_MENU, self._PyHocaGUI.OnShareCustomLocalFolder, id=ID_SHARECUSTOMLOCALFOLDER)
        self._PyHocaGUI.Bind(wx.EVT_MENU, self._PyHocaGUI.OnRememberSharedFolders, id=ID_REMEMBERSHAREDFOLDERS)
        self._PyHocaGUI.Bind(wx.EVT_MENU, self._PyHocaGUI.OnUnshareAllLocalFolders, id=ID_UNSHAREALLLOCALFOLDERS)


class PyHocaGUI_Menu_LaunchSingleApplication(wx.Menu):
    """\
    Submenu that triggers single application launches.

    """
    def __init__(self, _PyHocaGUI, caller=None, profile_name=None):
        """\
        Single application launching submenu (constructor).

        @param _PyHocaGUI: main application instance
        @type _PyHocaGUI: C{obj}
        @param caller: unused
        @type caller: C{None}
        @param profile_name: session profile name this submenu is for
        @type profile_name: C{str}

        """
        self._PyHocaGUI = _PyHocaGUI
        self._pyhoca_logger = self._PyHocaGUI._pyhoca_logger

        wx.Menu.__init__(self)

        _available_applications = {
            'WWWBROWSER': _('Internet Browser'),
            'MAILCLIENT': _('Email Client'),
            'OFFICE': _('Office'),
            'TERMINAL': _('Terminal'),
            }

        for application in self._PyHocaGUI.get_profile_config(profile_name, 'applications'):

            _app_id = wx.NewId()
            self._PyHocaGUI._eventid_profilenames_map[_app_id] = profile_name
            self._PyHocaGUI._eventid_applications_map[_app_id] = application
            self.Append(_app_id, _available_applications[application])
            self._PyHocaGUI.Bind(wx.EVT_MENU, self._PyHocaGUI.OnApplicationStart, id=_app_id)

    def AppendMenu(self, *args, **kwargs):
        return _legacy_wrap_AppendMenu(self, *args, **kwargs)


def _generate_Menu_PublishedApplications(_PyHocaGUI, caller=None, profile_name=None, session_name=None):
    """\
    Generate wxPython based menu tree for X2Go published applications.

    @param _PyHocaGUI: main application instance
    @type _PyHocaGUI: C{obj}
    @param caller: unused
    @type caller: C{None}
    @param profile_name: session profile name this submenu is for
    @type profile_name: C{str}
    @param session_name: X2Go session name this submenu is for
    @type session_name: C{str}

    @return: dictionary based menu tree containing wx.Menu objects for menu rendering.
    @rtype: C{dict}

    """
    _lang = _PyHocaGUI.lang
    _pubapp_session = _PyHocaGUI.get_session_of_session_name(session_name, return_object=True, match_profile_name=profile_name)
    try:
        menu_map = _pubapp_session.get_published_applications(lang=_lang, max_no_submenus=_PyHocaGUI.args.published_applications_no_submenus)
    except AttributeError:
        menu_map = None
    if not menu_map or _lang not in menu_map:
        menu_map = { _lang: {} }

    if x2go.defaults.X2GOCLIENT_OS == 'Windows':
        _icon_size = "16x16"
    else:
        _icon_size = "22x22"

    _icons_location = basepath.icons_basepath
    _category_name_translator = {
        'Multimedia': (_('Multimedia'), os.path.normpath('%s/PyHoca/%s/applications-multimedia.png' % (_icons_location, _icon_size), ), ),
        'Development': (_('Development'), os.path.normpath('%s/PyHoca/%s/applications-development.png' % (_icons_location, _icon_size), ), ),
        'Education': (_('Education'), os.path.normpath('%s/PyHoca/%s/applications-education.png' % (_icons_location, _icon_size), ), ),
        'Games': (_('Games'), os.path.normpath('%s/PyHoca/%s/applications-games.png' % (_icons_location, _icon_size), ), ),
        'Graphics': (_('Graphics'), os.path.normpath('%s/PyHoca/%s/applications-graphics.png' % (_icons_location, _icon_size), ), ),
        'Internet': (_('Internet'), os.path.normpath('%s/PyHoca/%s/applications-internet.png' % (_icons_location, _icon_size), ), ),
        'Office': (_('Office Applications'), os.path.normpath('%s/PyHoca/%s/applications-office.png' % (_icons_location, _icon_size), ), ),
        'System': (_('System'), os.path.normpath('%s/PyHoca/%s/applications-system.png' % (_icons_location, _icon_size), ), ),
        'Utilities': (_('Utilities'), os.path.normpath('%s/PyHoca/%s/applications-utilities.png' % (_icons_location, _icon_size), ), ),
        'Other Applications': (_('Other Applications'), os.path.normpath('%s/PyHoca/%s/applications-other.png' % (_icons_location, _icon_size), ), ),
        'TOP': ('TOP', os.path.normpath('%s/PyHoca/%s/x2go-logo-grey.png' % (_icons_location, _icon_size), ), ),
    }

    _PyHocaGUI._eventid_pubapp_execmap[profile_name] = {}

    nolog = wx.LogNull()
    _wx_menu_map = {}
    if list(menu_map[_lang].keys()):
        for cat in list(menu_map[_lang].keys()):

            _wx_menu_map[_category_name_translator[cat][0]] = (wx.Menu(), _category_name_translator[cat][1])

            for _item in menu_map[_lang][cat]:

                _pubapp_id = wx.NewId()
                _PyHocaGUI._eventid_profilenames_map[_pubapp_id] = profile_name
                _PyHocaGUI._eventid_sessionnames_map[_pubapp_id] = session_name
                _PyHocaGUI._eventid_pubapp_execmap[profile_name][_pubapp_id] = _item['exec']

                _menu_item = wx.MenuItem(_wx_menu_map[_category_name_translator[cat][0]][0], id=_pubapp_id, text=_item['name'], help=_item['comment'])
                if not _item['icon']:
                    _menu_item.SetBitmap(wx.Bitmap(os.path.normpath('%s/PyHoca/%s/x2go-logo-grey.png' % (_icons_location, _icon_size))))
                else:
                    _menu_entry_icon_decoded = base64.b64decode(_item['icon'])
                    _icon_image = wx.ImageFromStream(io.StringIO(_menu_entry_icon_decoded))
                    _icon_bitmap = None
                    try:
                        if x2go.defaults.X2GOCLIENT_OS == 'Windows':
                            _icon_bitmap = wx.BitmapFromImage(_icon_image.Scale(16,16))
                        else:
                            _icon_bitmap = wx.BitmapFromImage(_icon_image.Scale(22,22))
                    except:
                        # catch: wx._core.PyAssertionError: C++ assertion "IsOk()" failed at
                        # ../src/common/image.cpp(429) in Scale(): invalid image
                        continue

                    if _icon_bitmap:
                        _menu_item.SetBitmap(_icon_bitmap)

                _wx_menu_map[_category_name_translator[cat][0]][0].AppendItem(_menu_item)
                _PyHocaGUI.Bind(wx.EVT_MENU, _PyHocaGUI.OnPubAppExecution, id=_pubapp_id)

    del nolog
    return _wx_menu_map


class PyHocaGUI_Menu_TaskbarSessionProfile(wx.Menu):
    """\
    Submenu for a connected session profile.

    """
    def __init__(self, _PyHocaGUI, caller=None, profile_name=None):
        """\
        Session profile submenu (constructor).

        @param _PyHocaGUI: main application instance
        @type _PyHocaGUI: C{obj}
        @param caller: unused
        @type caller: C{None}
        @param profile_name: session profile name this submenu is for
        @type profile_name: C{str}

        """
        self._PyHocaGUI = _PyHocaGUI
        self._pyhoca_logger = self._PyHocaGUI._pyhoca_logger

        wx.Menu.__init__(self)

        ID_AUTHENTICATE_BROKER=wx.NewId()
        ID_CONNECT=wx.NewId()
        ID_PUBAPPSESSIONSTART=wx.NewId()
        ID_SESSIONSTART=wx.NewId()
        ID_SHADOWSESSIONSTART=wx.NewId()
        ID_LAUNCHAPPLICATION = wx.NewId()
        ID_CLEANSESSIONS = wx.NewId()
        ID_EDITPROFILEWHILECONNECTED = wx.NewId()
        ID_SHARELOCALFOLDER = wx.NewId()
        ID_UNSHAREFOLDERS = wx.NewId()

        _foldersharing_disabled = False

        if self._PyHocaGUI.with_brokerage and not self._PyHocaGUI.session_profiles.is_broker_authenticated():
            _auth_menu_text = _('Connect to') + self._PyHocaGUI.broker_name
            self.Append(ID_AUTHENTICATE_BROKER, _auth_menu_text)
            self._PyHocaGUI.Bind(wx.EVT_MENU, self._PyHocaGUI.OnBrokerAuthenticate, id=ID_AUTHENTICATE_BROKER)

        elif self._PyHocaGUI.args.single_session_profile and not self._PyHocaGUI.is_session_profile(profile_name):
            connect = self.Append(ID_CONNECT, _('Connect %s') % profile_name)
            connect.Enable(False)
        else:
            _applications = self._PyHocaGUI.get_profile_config(profile_name, 'applications')
            _command = self._PyHocaGUI.get_profile_config(profile_name, 'command')
            _published = self._PyHocaGUI.get_profile_config(profile_name, 'published')
            _useexports = self._PyHocaGUI.get_profile_config(profile_name, 'useexports')

            if profile_name in self._PyHocaGUI._temp_disabled_profile_names:
                _connecting_info = self.Append(wx.NewId(), _('Currently connecting...'))
                _connecting_info.Enable(False)

            elif self._PyHocaGUI.args.single_session_profile and \
                 not self._PyHocaGUI.is_profile_connected(profile_name=profile_name):
                    self._PyHocaGUI._eventid_profilenames_map[ID_CONNECT] = profile_name
                    self.Append(ID_CONNECT, _('Connect %s') % profile_name)
                    self._PyHocaGUI.Bind(wx.EVT_MENU, self._PyHocaGUI.OnSessionAuthenticate, id=ID_CONNECT)

            else:

                self._PyHocaGUI._eventid_profilenames_map[ID_SESSIONSTART] = \
                    self._PyHocaGUI._eventid_profilenames_map[ID_SHADOWSESSIONSTART] = profile_name

                if _command in x2go.defaults.X2GO_DESKTOPSESSIONS:
                    self.Append(ID_SESSIONSTART, '%s (%s)' % (_("Start &new Desktop Session"), _command))
                    self._PyHocaGUI.Bind(wx.EVT_MENU, self._PyHocaGUI.OnSessionStart, id=ID_SESSIONSTART)

                elif _command == 'SHADOW':
                    self.Append(ID_SHADOWSESSIONSTART, _("Start Desktop Sharing Session"))
                    self._PyHocaGUI.Bind(wx.EVT_MENU, self._PyHocaGUI.OnShadowSessionStart, id=ID_SHADOWSESSIONSTART)

                elif _command == '' and _published:
                    _pub_app_start_item = None
                    if profile_name in self._PyHocaGUI._temp_launching_pubapp_profiles:
                        _pub_app_start_item = self.Append(ID_PUBAPPSESSIONSTART, _("Retrieving Application Menu..."))
                        _pub_app_start_item.Enable(False)
                    elif not (self._PyHocaGUI.disconnect_on_suspend and self._PyHocaGUI.disconnect_on_terminate):
                        self._PyHocaGUI._eventid_profilenames_map[ID_PUBAPPSESSIONSTART] = profile_name
                        _pub_app_start_item = self.Append(ID_PUBAPPSESSIONSTART, _("Retrieve Application Menu"))
                        self._PyHocaGUI.Bind(wx.EVT_MENU, self._PyHocaGUI.OnPubAppSessionStart, id=ID_PUBAPPSESSIONSTART)
                elif _command == 'RDP':
                    self.Append(ID_SESSIONSTART, _("Start &new RDP Session"))
                    self._PyHocaGUI.Bind(wx.EVT_MENU, self._PyHocaGUI.OnSessionStart, id=ID_SESSIONSTART)
                else:
                    self.Append(ID_SESSIONSTART, _("Start &new Session"))
                    self._PyHocaGUI.Bind(wx.EVT_MENU, self._PyHocaGUI.OnSessionStart, id=ID_SESSIONSTART)

                if _command == '' and _published:

                    _pubapp_sessions = [ _pas for _pas in self._PyHocaGUI.client_pubapp_sessions_of_profile_name(profile_name, return_objects=True) if _pas.is_running() ]
                    if _pubapp_sessions:
                        _pubapp_session = _pubapp_sessions[0]
                        if _pub_app_start_item is not None:
                            _pub_app_start_item.Enable(False)
                        _foldersharing_disabled = _session_name_disabled = self._PyHocaGUI.is_session_name_disabled(profile_name, _pubapp_session.get_session_name())
                        _category_map = _generate_Menu_PublishedApplications(self._PyHocaGUI, caller=self, profile_name=profile_name, session_name=_pubapp_session.get_session_name())
                        _category_names = list(_category_map.keys())
                        _category_names.sort()
                        if (not self._PyHocaGUI.restricted_trayicon and not (self._PyHocaGUI.disconnect_on_suspend and self._PyHocaGUI.disconnect_on_terminate)) or (profile_name in self._PyHocaGUI._temp_launching_pubapp_profiles and _category_names):
                            self.AppendSeparator()
                        for cat_name in [ _cn for _cn in _category_names if _cn != 'TOP' ]:
                            _submenu = self.AppendMenu(wx.NewId(), cat_name, _category_map[cat_name][0])
                            _submenu.SetBitmap(wx.Bitmap(_category_map[cat_name][1]))
                            if _session_name_disabled:
                                _submenu.Enable(False)
                        if 'TOP' in _category_names:
                            for _menu_item in _category_map['TOP'][0].GetMenuItems():
                                _item = self.AppendItem(item=_menu_item)
                                if _session_name_disabled:
                                    _item.Enable(False)
                        if _category_names:
                            self.AppendSeparator()

                        ID_RESUMESESSION = wx.NewId()
                        ID_REFRESHMENU = wx.NewId()
                        ID_SUSPENDSESSION = wx.NewId()
                        ID_TERMINATESESSION = wx.NewId()
                        self._PyHocaGUI._eventid_profilenames_map[ID_RESUMESESSION] = \
                            self._PyHocaGUI._eventid_profilenames_map[ID_REFRESHMENU] = \
                            self._PyHocaGUI._eventid_profilenames_map[ID_SUSPENDSESSION] = \
                            self._PyHocaGUI._eventid_profilenames_map[ID_TERMINATESESSION] = profile_name
                        self._PyHocaGUI._eventid_sessionnames_map[ID_RESUMESESSION] = \
                            self._PyHocaGUI._eventid_sessionnames_map[ID_REFRESHMENU] = \
                            self._PyHocaGUI._eventid_sessionnames_map[ID_SUSPENDSESSION] = \
                            self._PyHocaGUI._eventid_sessionnames_map[ID_TERMINATESESSION] = _pubapp_session.get_session_name()

                        if _pubapp_session.is_running():
                            _refresh_menu_item = self.Append(ID_REFRESHMENU, _("Refresh menu tree"))
                            self.AppendSeparator()
                            if self._PyHocaGUI.disconnect_on_suspend and self._PyHocaGUI.exit_on_disconnect and _pubapp_session.has_terminal_session():
                                _suspend_item = self.Append(ID_SUSPENDSESSION, _("Suspend Session (and disconnect/exit)"))
                            elif self._PyHocaGUI.disconnect_on_suspend and _pubapp_session.has_terminal_session():
                                _suspend_item = self.Append(ID_SUSPENDSESSION, _("Suspend Session (and disconnect)"))
                            else:
                                _suspend_item = self.Append(ID_SUSPENDSESSION, _("Suspend Session"))
                            if _session_name_disabled:
                                _refresh_menu_item.Enable(False)
                                _suspend_item.Enable(False)
                        elif _pubapp_session.is_suspended():
                            _resume_item = self.Append(ID_RESUMESESSION, _("Resume Session"))
                            if _session_name_disabled:
                                _resume_item.Enable(False)
                        if self._PyHocaGUI.disconnect_on_terminate and self._PyHocaGUI.exit_on_disconnect and _pubapp_session.has_terminal_session():
                            _terminate_item = self.Append(ID_TERMINATESESSION, _("Terminate Session (and disconnect/exit)"))
                        elif self._PyHocaGUI.disconnect_on_terminate and _pubapp_session.has_terminal_session():
                            _terminate_item = self.Append(ID_TERMINATESESSION, _("Terminate Session (and disconnect)"))
                        else:
                            _terminate_item = self.Append(ID_TERMINATESESSION, _("Terminate Session"))
                        if _session_name_disabled:
                            _terminate_item.Enable(False)

                        self._PyHocaGUI.Bind(wx.EVT_MENU, self._PyHocaGUI.OnSessionResume, id=ID_RESUMESESSION)
                        self._PyHocaGUI.Bind(wx.EVT_MENU, self._PyHocaGUI.OnPubAppRefreshMenu, id=ID_REFRESHMENU)
                        self._PyHocaGUI.Bind(wx.EVT_MENU, self._PyHocaGUI.OnSessionSuspend, id=ID_SUSPENDSESSION)
                        self._PyHocaGUI.Bind(wx.EVT_MENU, self._PyHocaGUI.OnSessionTerminate, id=ID_TERMINATESESSION)

                else:

                    # preparing profile_name information for the main PyHocaGUI instance
                    self._PyHocaGUI._eventid_profilenames_map[ID_LAUNCHAPPLICATION] = \
                        self._PyHocaGUI._eventid_profilenames_map[ID_CLEANSESSIONS] = profile_name

                    if _applications and _command in list(x2go.defaults.X2GO_DESKTOPSESSIONS.keys()) and not _published:
                        self.AppendSeparator()
                        self.AppendMenu(ID_LAUNCHAPPLICATION,
                                        _("Launch Single Application"),
                                        PyHocaGUI_Menu_LaunchSingleApplication(self._PyHocaGUI, caller=self, profile_name=profile_name)
                                       )
                        if _command != 'SHADOW' and not self._PyHocaGUI.restricted_trayicon:
                            self.Append(ID_SHADOWSESSIONSTART, _("Start Desktop Sharing Session"))
                            self._PyHocaGUI.Bind(wx.EVT_MENU, self._PyHocaGUI.OnShadowSessionStart, id=ID_SHADOWSESSIONSTART)

                    if _published:

                        if not self._PyHocaGUI.restricted_trayicon:
                            self.AppendSeparator()

                        _pubapp_session = None
                        _pubapp_sessions = [ _pas for _pas in self._PyHocaGUI.client_pubapp_sessions_of_profile_name(profile_name, return_objects=True) if _pas.is_running() ]
                        if _pubapp_sessions:
                            _pubapp_session = _pubapp_sessions[0]
                        if _pubapp_session and _pubapp_session.is_running():
                            _session_name_disabled = self._PyHocaGUI.is_session_name_disabled(profile_name, _pubapp_session.get_session_name())
                            _category_map = _generate_Menu_PublishedApplications(self._PyHocaGUI, caller=self, profile_name=profile_name, session_name=_pubapp_session.get_session_name())
                            _category_names = list(_category_map.keys())
                            _category_names.sort()
                            for cat_name in [ _cn for _cn in _category_names if _cn != 'TOP' ]:
                                _submenu = self.AppendMenu(wx.NewId(), cat_name, _category_map[cat_name][0])
                                _submenu.SetBitmap(wx.Bitmap(_category_map[cat_name][1]))
                                if _session_name_disabled:
                                    _submenu.Enable(False)
                            if 'TOP' in _category_names:
                                for _menu_item in _category_map['TOP'][0].GetMenuItems():
                                    _item = self.AppendItem(item=_menu_item)
                                    if _session_name_disabled:
                                        _item.Enable(False)

                            self.AppendSeparator()

                            _marker = ''
                            _status = None
                            if _pubapp_session.is_master_session(): _marker = '(*)'
                            if _pubapp_session.is_running(): _status = 'R'
                            elif _pubapp_session.is_suspended(): _status = 'S'

                            if _status:
                                _submenu = self.AppendMenu(wx.NewId(),
                                                           _('Manage Application Menu')+' %s' % _marker,
                                                           PyHocaGUI_Menu_TaskbarSessionActions(self._PyHocaGUI, caller=self,
                                                                                                profile_name=profile_name,
                                                                                                session_name=_pubapp_session.get_session_name(),
                                                                                                status=_status,
                                                                                               )
                                                          )
                                if _session_name_disabled:
                                    _submenu.Enable(False)

                        else:
                            self._PyHocaGUI._eventid_profilenames_map[ID_PUBAPPSESSIONSTART] = profile_name
                            if profile_name in self._PyHocaGUI._temp_launching_pubapp_profiles:
                                _ram = self.Append(ID_PUBAPPSESSIONSTART, _("Retrieving Application Menu..."))
                                _ram.Enable(False)
                            else:
                                self.Append(ID_PUBAPPSESSIONSTART, _("Retrieve Application Menu"))
                            self._PyHocaGUI.Bind(wx.EVT_MENU, self._PyHocaGUI.OnPubAppSessionStart, id=ID_PUBAPPSESSIONSTART)

                    _query_session_uuid = self._PyHocaGUI.client_connected_sessions_of_profile_name(profile_name, return_objects=False)[0]
                    _session_list = self._PyHocaGUI.list_sessions(_query_session_uuid)

                    _session_list_names = []
                    if _session_list:

                        # newest sessions at the top
                        if _published:
                            _session_list_names = [ _s_name for _s_name in list(_session_list.keys()) if not _session_list[_s_name].is_published_applications_provider() ]
                        else:
                            _session_list_names = list(_session_list.keys())
                        _session_list_names.reverse()

                    if _session_list_names:

                        _session_list_matching_profile = []
                        for session_name in _session_list_names:

                            session = self._PyHocaGUI.get_session_of_session_name(session_name, return_object=True, match_profile_name=profile_name)

                            if not session:
                                continue
                            else:
                                _session_list_matching_profile.append(session)

                        if _session_list_matching_profile:

                            self.AppendSeparator()

                            for session in _session_list_matching_profile:

                                session_name = session.get_session_name()

                                _s_id = wx.NewId()

                                if _session_list[session_name].get_status() == 'R':
                                    state = _('Running')
                                elif _session_list[session_name].get_status() == 'S':
                                    state = _('Suspended')
                                _marker = ''
                                if session and session.is_master_session():
                                    _marker = '(*)'
                                if session:
                                    session_submenu = self.Append(_s_id,
                                                                  '%s: »%s« %s' % (state, session_name, _marker),
                                                                  PyHocaGUI_Menu_TaskbarSessionActions(self._PyHocaGUI, caller=self,
                                                                                                       profile_name=profile_name,
                                                                                                       session_name=session_name,
                                                                                                       session_info=_session_list[session_name],
                                                                                                      )
                                                                 )
                                    if profile_name in self._PyHocaGUI._temp_disabled_session_names and session_name in self._PyHocaGUI._temp_disabled_session_names[profile_name]:
                                        session_submenu.Enable(False)

                            # redefine list of session names to decide if the clean all session menu item is not be shown
                            _session_list_names = [ _s.get_session_name() for _s in _session_list_matching_profile if not _session_list[_s.get_session_name()].is_published_applications_provider() ]

                            if _session_list_names:
                                self.Append(ID_CLEANSESSIONS, _("&Clean all sessions"))
                                self._PyHocaGUI.Bind(wx.EVT_MENU, self._PyHocaGUI.OnCleanSessions, id=ID_CLEANSESSIONS)

                if not self._PyHocaGUI.restricted_trayicon:

                    self.AppendSeparator()
                    if self._PyHocaGUI.session_profiles.is_mutable(profile_name):
                        self.Append(ID_EDITPROFILEWHILECONNECTED, _("Customize &profile"))
                    else:
                        self.Append(ID_EDITPROFILEWHILECONNECTED, _("View &profile"))
                    self._PyHocaGUI.Bind(wx.EVT_MENU, self._PyHocaGUI.OnProfileEditWhileConnected, id=ID_EDITPROFILEWHILECONNECTED)

                self._PyHocaGUI._eventid_profilenames_map[ID_EDITPROFILEWHILECONNECTED] = \
                    self._PyHocaGUI._eventid_profilenames_map[ID_SHARELOCALFOLDER] = \
                    self._PyHocaGUI._eventid_profilenames_map[ID_UNSHAREFOLDERS] = profile_name

                if _useexports and \
                   self._PyHocaGUI._X2GoClient__profile_is_folder_sharing_available(profile_name=profile_name) and \
                   self._PyHocaGUI.get_master_session(profile_name) is not None and \
                   self._PyHocaGUI.is_session_name_enabled(profile_name, self._PyHocaGUI.get_master_session(profile_name).get_session_name()):

                    if self._PyHocaGUI.restricted_trayicon:
                        self.AppendSeparator()

                    _shared_folders = self.AppendMenu(ID_SHARELOCALFOLDER,
                                                      _("Shared &folders"),
                                                      PyHocaGUI_Menu_TaskbarProfileSharedFolders(self._PyHocaGUI, caller=self, profile_name=profile_name)
                                                     )
                    if not self._PyHocaGUI.get_master_session(profile_name=profile_name) or _foldersharing_disabled:
                        _shared_folders.Enable(False)


        if profile_name in self._PyHocaGUI.client_connected_profiles(return_profile_names=True) and not self._PyHocaGUI.restricted_trayicon:
            self.AppendSeparator()
            ID_SERVERINFO = wx.NewId()
            self._PyHocaGUI._eventid_profilenames_map[ID_SERVERINFO] = profile_name
            self.Append(ID_SERVERINFO, _("Server Information"))
            self._PyHocaGUI.Bind(wx.EVT_MENU, self._PyHocaGUI.OnViewServerInformation, id=ID_SERVERINFO)

        if profile_name in self._PyHocaGUI.client_connected_profiles(return_profile_names=True) and not self._PyHocaGUI.exit_on_disconnect:
            self.AppendSeparator()
            ID_DISCONNECT = wx.NewId()
            self._PyHocaGUI._eventid_profilenames_map[ID_DISCONNECT] = profile_name
            self.Append(ID_DISCONNECT, _("&Disconnect from Server"))
            self._PyHocaGUI.Bind(wx.EVT_MENU, self._PyHocaGUI.OnServerDisconnect, id=ID_DISCONNECT)

        if self._PyHocaGUI.args.single_session_profile:
            ID_EXIT = wx.NewId()
            if self._PyHocaGUI.client_running_sessions_of_profile_name(profile_name=self._PyHocaGUI.args.session_profile) and self._PyHocaGUI.exit_on_disconnect and not self._PyHocaGUI.disconnect_on_suspend:
                self.AppendSeparator()
                self.Append(ID_EXIT, _("Suspend Session and E&xit application"))
                self._PyHocaGUI.Bind(wx.EVT_MENU, self._PyHocaGUI.OnTaskbarExit, id=ID_EXIT)
            elif self._PyHocaGUI.is_profile_connected(profile_name=self._PyHocaGUI.args.session_profile) and self._PyHocaGUI.exit_on_disconnect and not self._PyHocaGUI.disconnect_on_suspend:
                self.AppendSeparator()
                self.Append(ID_EXIT, _("Disconnect and E&xit application"))
                self._PyHocaGUI.Bind(wx.EVT_MENU, self._PyHocaGUI.OnTaskbarExit, id=ID_EXIT)
            elif not self._PyHocaGUI.exit_on_disconnect and not (self._PyHocaGUI.disconnect_on_suspend or self._PyHocaGUI.disconnect_on_terminate):
                self.AppendSeparator()
                self.Append(ID_EXIT, _("E&xit"))
                self._PyHocaGUI.Bind(wx.EVT_MENU, self._PyHocaGUI.OnTaskbarExit, id=ID_EXIT)

    def AppendMenu(self, *args, **kwargs):
        return _legacy_wrap_AppendMenu(self, *args, **kwargs)


class PyHocaGUI_Menu_TaskbarProfileNames(wx.Menu):
    """\
    Render a (recursive) menu subtree that contains a cascaded menu tree of all session profile names.

    If session profile names contain '/' as a separator character then those session profile names
    will be rendered in a tree-like fashion.

    """
    def __init__(self, _PyHocaGUI, caller=None,
                 sub_profile_items=[], filter_profiles=[], disabled_profiles=[],
                 bind_method=None,
                 submenu=None,
                 group_menus=True, group_name='', parent_group=''):
        """\
        Session profile name (recursive) menu subtree

        @param _PyHocaGUI: main application instance
        @type _PyHocaGUI: C{obj}
        @param caller: unused
        @type caller: C{None}
        @param sub_profile_items: on recursion this argument contains those session profile names
            that will be rendered for this session profile subtree
        @type  sub_profile_items: C{list}
        @param filter_profiles: allow filtering of profile names (hiding certain session profiles)
        @type filter_profiles: C{list}
        @param disabled_profiles: session profile names that get greyed out in the menu tree
        @type disabled_profiles: C{list}
        @param bind_method: if session profile names are menu items this argument names the Python method that menu items will bind to
        @type bind_method: C{method}
        @param submenu: if session profile names are submenus this argument names the wx.Menu class that handles the submenu rendering
        @type submenu: C{class}
        @param group_menus: group session profile names in submenus (i.e. switch on recursion mode)
        @type group_menus: C{bool}
        @param group_name: on recursion, this argument names the parent menu folder of the current submenu
        @type group_name: C{str}
        @param parent_group: on recursion, the parent group of C{group_name}
        @type parent_group: C{str}

        """
        self._PyHocaGUI = _PyHocaGUI
        self._pyhoca_logger = self._PyHocaGUI._pyhoca_logger

        wx.Menu.__init__(self)

        if type(caller) == PyHocaGUI_Menu_TaskbarOptionsManager and self._PyHocaGUI.session_profiles.supports_mutable_profiles():
            ID_ADDPROFILE = wx.NewId()
            self.Append(ID_ADDPROFILE, _("Add Profile"))
            self._PyHocaGUI.Bind(wx.EVT_MENU, self._PyHocaGUI.OnProfileAdd, id=ID_ADDPROFILE)

            self.AppendSeparator()

        if sub_profile_items:
            _profile_names = sub_profile_items
        else:
            _profile_names = self._PyHocaGUI.session_profiles.profile_names
        _profile_names.sort()

        _profile_groups = []
        if group_menus:

            _parent_group = group_name

            # grouping of session profile menus
            for profile_name in _profile_names:
                if len(profile_name.split('/')) >= 2:
                    _group_name = profile_name.split('/')[0]
                    if not _group_name in _profile_groups:
                        _profile_groups.append(_group_name)

            _profile_groups.sort()

            for profile_group in _profile_groups:
                _sub_profile_items = []
                for profile_name in [ p for p in _profile_names if p.startswith('%s/' % profile_group) ]:
                    _sub_profile_name = "/".join(profile_name.split('/')[1:])
                    _sub_profile_items.append(_sub_profile_name)
                    filter_profiles.append(profile_name)

                _this_id = wx.NewId()
                self.AppendMenu(_this_id,
                                profile_group,
                                PyHocaGUI_Menu_TaskbarProfileNames(self._PyHocaGUI,
                                                                   caller=self,
                                                                   sub_profile_items=_sub_profile_items,
                                                                   filter_profiles=[],
                                                                   disabled_profiles=disabled_profiles,
                                                                   submenu=submenu,
                                                                   bind_method=bind_method,
                                                                   group_name=profile_group,
                                                                   parent_group=_parent_group,
                                                                   group_menus=True
                                                                  )
                               )
        if filter_profiles:
            _profile_names = [ p for p in _profile_names if p not in filter_profiles ]

        for profile_name in _profile_names:
            if group_name:
                if parent_group:
                    _real_profile_name = '%s/%s/%s' % (parent_group, group_name, profile_name)
                else:
                    _real_profile_name = '%s/%s' % (group_name, profile_name)
                _show_profile_name = profile_name
            else:
                _real_profile_name = profile_name
                _show_profile_name = profile_name
            _this_id = wx.NewId()
            self._PyHocaGUI._eventid_profilenames_map[_this_id] = _real_profile_name
            _menu_profile_name = self._PyHocaGUI.show_profile_metatypes and '%s (%s)' % (_show_profile_name, self._PyHocaGUI.get_profile_metatype(_real_profile_name)) or _show_profile_name
            if submenu is not None:
                _sub = self.AppendMenu(_this_id, _menu_profile_name, submenu(self._PyHocaGUI, caller=self, profile_name=_real_profile_name))
                if disabled_profiles and _real_profile_name in disabled_profiles:
                    _sub.Enable(False)
            else:
                _item = self.Append(_this_id, _menu_profile_name)

                if disabled_profiles and _real_profile_name in disabled_profiles:
                    _item.Enable(False)
                if bind_method is not None:
                    self._PyHocaGUI.Bind(wx.EVT_MENU, bind_method, id=_this_id)
                    self._PyHocaGUI.Bind(wx.EVT_UPDATE_UI, self.OnUpdateUI, id=_this_id)

        if not group_name and (not _profile_groups and not _profile_names) and not filter_profiles:
            if self._PyHocaGUI.with_brokerage:
                _dummy = self.Append(wx.NewId(), _('Session broker is not connected'))
            else:
                _dummy = self.Append(wx.NewId(), _('No session profiles defined'))
            _dummy.Enable(False)

        else:
            if bind_method is None:
                self.AppendSeparator()
                _export_id = wx.NewId()
                _export_group_name = "%s/%s" % (parent_group, group_name)
                _export_group_name = _export_group_name.strip("/")
                self._PyHocaGUI._eventid_exportprofiles_map[_export_id] = _export_group_name
                if not group_name:
                    self.Append(_export_id, _('Export all Profiles'))
                else:
                    self.Append(_export_id, _('Export Profile Group'))
                self._PyHocaGUI.Bind(wx.EVT_MENU, self._PyHocaGUI.OnProfileExport, id=_export_id)

        if bind_method is None and not group_name and self._PyHocaGUI.session_profiles.supports_mutable_profiles():
            _import_id = wx.NewId()
            self.AppendSeparator()
            self.Append(_import_id, _('Import Session Profiles'))
            self._PyHocaGUI.Bind(wx.EVT_MENU, self._PyHocaGUI.OnProfileImport, id=_import_id)

    def OnUpdateUI(self, evt):
        profile_name = self._PyHocaGUI._eventid_profilenames_map[evt.GetId()]
        if profile_name in self._PyHocaGUI._temp_disabled_profile_names:
            self._pyhoca_logger('Updating UI, temporarily disabling session profile %s' % profile_name)
            self.Enable(id=evt.GetId(), enable=False)
        elif profile_name not in self._PyHocaGUI._temp_disabled_profile_names and profile_name not in self._PyHocaGUI.client_connected_sessions(return_profile_names=True):
            self._pyhoca_logger('Updating UI, re-enabling session profile %s' % profile_name)
            self.Enable(id=evt.GetId(), enable=True)

    def AppendMenu(self, *args, **kwargs):
        return _legacy_wrap_AppendMenu(self, *args, **kwargs)


class PyHocaGUI_Menu_TaskbarSessionManager(wx.Menu):
    """\
    Right-click menu of the L{PyHocaGUI} systray icon.

    """
    def __init__(self, _PyHocaGUI, caller=None):
        """\
        Session management menu of L{PyHocaGUI} (constructor).

        @param _PyHocaGUI: main application instance
        @type _PyHocaGUI: C{obj}
        @param caller: unused
        @type caller: C{None}

        """
        self._PyHocaGUI = _PyHocaGUI
        self._pyhoca_logger = self._PyHocaGUI._pyhoca_logger

        wx.Menu.__init__(self)

        ID_AUTHENTICATE_SESSION = wx.NewId()
        ID_AUTHENTICATE_BROKER = wx.NewId()
        ID_EXIT = wx.NewId()

        if self._PyHocaGUI.with_brokerage and not self._PyHocaGUI.session_profiles.is_broker_authenticated():
            _auth_menu_text = _('Connect to') + ' ' + self._PyHocaGUI.broker_name
            self.Append(ID_AUTHENTICATE_BROKER, _auth_menu_text)
            self._PyHocaGUI.Bind(wx.EVT_MENU, self._PyHocaGUI.OnBrokerAuthenticate, id=ID_AUTHENTICATE_BROKER)
            self.AppendSeparator()
        else:
            _auth_menu_text = _('Connect Server')
            self.AppendMenu(ID_AUTHENTICATE_SESSION,
                            _auth_menu_text,
                            PyHocaGUI_Menu_TaskbarProfileNames(self._PyHocaGUI,
                                                               caller=self,
                                                               filter_profiles=[],
                                                               disabled_profiles=self._PyHocaGUI.client_connected_profiles(return_profile_names=True) + self._PyHocaGUI._temp_disabled_profile_names,
                                                               bind_method=self._PyHocaGUI.OnSessionAuthenticate,
                                                              )
                           )
            self.AppendSeparator()

        _profile_names = self._PyHocaGUI.session_profiles.profile_names
        _profile_names.sort()

        _connected_sessions = False
        for profile_name in _profile_names:
            if profile_name in self._PyHocaGUI._X2GoClient__client_connected_sessions(return_profile_names=True):
                _connected_sessions = True
                _this_id = wx.NewId()

                _menu_profile_name = self._PyHocaGUI.show_profile_metatypes and '%s (%s)' % (profile_name, self._PyHocaGUI.get_profile_metatype(profile_name)) or profile_name
                try:
                    self.AppendMenu(_this_id,
                                    _menu_profile_name,
                                    PyHocaGUI_Menu_TaskbarSessionProfile(self._PyHocaGUI, caller=self, profile_name=profile_name)
                                   )
                except x2go.x2go_exceptions.X2GoSessionRegistryException:
                    pass

        if _connected_sessions:
            self.AppendSeparator()

        self.Append(ID_EXIT, _("E&xit"))
        self._PyHocaGUI.Bind(wx.EVT_MENU, self._PyHocaGUI.OnTaskbarExit, id=ID_EXIT)

    def AppendMenu(self, *args, **kwargs):
        return _legacy_wrap_AppendMenu(self, *args, **kwargs)



# -*- coding: utf-8 -*-
# vim:fenc=utf-8

# Copyright (C) 2010-2020 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
# Copyright (C) 2010-2016 by Dick Kniep <dick.kniep@lindix.nl>
#
# PyHoca GUI is free software; you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# PyHoca GUI is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program; if not, write to the
# Free Software Foundation, Inc.,
# 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.

import wx
import os

wx_major = int(wx.__version__.split('.')[0])
if wx_major < 4:
    from wx import SplashScreen as _SplashScreen
    _wx_SPLASH_CENTRE_ON_SCREEN = wx.SPLASH_CENTRE_ON_SCREEN
    _wx_SPLASH_TIMEOUT = wx.SPLASH_TIMEOUT
else:
    from wx.adv import SplashScreen as _SplashScreen
    _wx_SPLASH_CENTRE_ON_SCREEN = wx.adv.SPLASH_CENTRE_ON_SCREEN
    _wx_SPLASH_TIMEOUT = wx.adv.SPLASH_TIMEOUT

# PyHoca-GUI modules
from . import basepath

class PyHocaGUI_SplashScreen(_SplashScreen):
    """\
    L{PyHocaGUI} splash screen that gets shown an application startup.

    """
    def __init__(self, _PyHocaGUI, splash_image=None):
        """
        Splash screen (constructor).

        @param splash_image: file name of a splash image file (currently, only PNGs are supported)
        @type splash_image: C{str}

        """
        if splash_image:
            splash_image = os.path.expanduser(splash_image)

        if splash_image and os.path.basename(splash_image) == splash_image:
            splash_image = os.path.join(basepath.images_basepath, splash_image)

        if not os.path.isfile(str(splash_image)):
            splash_image = None

        if splash_image is None:
            splash_image = os.path.join(basepath.images_basepath, '{appname}_splash.png'.format(appname=_PyHocaGUI.appname))
            if not os.path.exists(splash_image):
                splash_image = os.path.join(basepath.images_basepath, 'PyHoca-GUI_splash.png')

        if os.path.isfile(str(splash_image)):
            splash_wximage = wx.Image(splash_image, wx.BITMAP_TYPE_PNG, )
            splash_wximage.Rescale(400, int(float(400)/splash_wximage.Width*splash_wximage.Height))

            splash_wxbitmap = splash_wximage.ConvertToBitmap()
            _SplashScreen.__init__(self,
                                   splash_wxbitmap,
                                   splashStyle=_wx_SPLASH_CENTRE_ON_SCREEN|_wx_SPLASH_TIMEOUT,
                                   milliseconds=5000,
                                   parent=None,
                                   style=wx.SIMPLE_BORDER|wx.STAY_ON_TOP|wx.FRAME_NO_TASKBAR
                                  )
            self.Bind(wx.EVT_CLOSE, self.OnClose)

    def OnClose(self, evt):
        """\
        Hide the splash screen.

        """
        # Make sure the default handler runs too so this window gets
        # destroyed
        evt.Skip()
        self.Hide()



# -*- coding: utf-8 -*-
# vim:fenc=utf-8

# Copyright (C) 2010-2020 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
# Copyright (C) 2010-2016 by Dick Kniep <dick.kniep@lindix.nl>
#
# PyHoca GUI is free software; you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# PyHoca GUI is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program; if not, write to the
# Free Software Foundation, Inc.,
# 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.

modules ={}

# Python X2Go
import x2go

import wx
import os

# PyHoca-GUI modules
# ... NONE ...

if 'DESKTOP_SESSION' in os.environ:
    WINDOW_MANAGER = os.environ['DESKTOP_SESSION']
else:
    WINDOW_MANAGER = 'generic'

class PyHocaGUI_DialogBoxServerInfo(wx.Dialog):
    """\
    Simple dialog box for showing server information.

    """
    def __init__(self, _PyHocaGUI, profile_name):
        """\
        Server information dialog box (constructor).

        @param _PyHocaGUI: the master/parent object of the application
        @type _PyHocaGUI: C{obj}
        @param profile_name: session profile name
        @type profile_name: C{str}

        """
        self._PyHocaGUI = _PyHocaGUI
        self._pyhoca_logger = self._PyHocaGUI._pyhoca_logger
        self._pyhoca_logger('server info box started', loglevel=x2go.loglevel_INFO, )

        self.current_profile_name = profile_name

        wx.Dialog.__init__(self, None, id=-1, title=profile_name, style=wx.DEFAULT_FRAME_STYLE, )
        self._PyHocaGUI._sub_windows.append(self)

        self.SetTitle(_('Server Information - %s') % profile_name)

        self.titleLbl = wx.StaticText(self, wx.ID_ANY, _('Session Profile: %s\n\nList of X2Go Server components, add-ons and their versions...') % self.current_profile_name, size=(-1, -1))

        self.infoArea = wx.TextCtrl(self, id=-1, value="", size=(520,300), style=wx.TE_READONLY|wx.TE_MULTILINE|wx.SUNKEN_BORDER)

        ID_REFRESH = wx.NewId()
        self.refreshBtn = wx.Button(self, ID_REFRESH, _('Refresh'), )
        self.cancelBtn = wx.Button(self, wx.ID_CANCEL, _('Close'), )

        self.Bind(wx.EVT_BUTTON, self.OnRefreshServerInfo, self.refreshBtn)
        self.Bind(wx.EVT_BUTTON, self.OnCancel, self.cancelBtn)

        titleSizer = wx.BoxSizer(wx.HORIZONTAL)
        infoSizer = wx.BoxSizer(wx.HORIZONTAL)
        btnSizer = wx.BoxSizer(wx.HORIZONTAL)
        mainSizer = wx.BoxSizer(wx.VERTICAL)

        titleSizer.Add(self.titleLbl, 0, wx.ALL, 5)

        infoSizer.Add(self.infoArea, 0, wx.ALL, 5)

        btnSizer.Add(self.refreshBtn, 0, wx.ALL, 5)
        btnSizer.Add(self.cancelBtn, 0, wx.ALL, 5)

        mainSizer.Add(titleSizer, 0, wx.ALL, 5)
        mainSizer.Add(infoSizer, 0, wx.ALL, 5)
        mainSizer.Add(btnSizer, 0, wx.ALL|wx.ALIGN_RIGHT, 5)

        self.SetSizerAndFit(mainSizer)
        self.Layout()

        maxX, maxY = wx.GetDisplaySize()

        # we will use the logon window position for this session re-titling windows, as well
        if self._PyHocaGUI.logon_window_position_x and self._PyHocaGUI.logon_window_position_y:

            # allow positioning of logon window via command line option
            if self._PyHocaGUI.logon_window_position_x < 0:
                move_x = maxX - (self.GetSize().GetWidth() + self._PyHocaGUI.logon_window_position_x)
            else:
                move_x = self._PyHocaGUI.logon_window_position_x
            if self._PyHocaGUI.logon_window_position_y < 0:
                move_y = maxX - (self.GetSize().GetHeight() + self._PyHocaGUI.logon_window_position_y)
            else:
                move_y = self._PyHocaGUI.logon_window_position_y

        elif (x2go.X2GOCLIENT_OS == 'Linux') and (WINDOW_MANAGER in ('gnome', 'gnome-fallback', 'awesome', 'mate', 'ubuntu', 'ubuntu-2d', 'openbox-gnome', )):

            # automatically place logon Window for GNOME, awesome
            move_x = maxX - (self.GetSize().GetWidth() + 20)
            move_y = 35

        else:

            # automatically place logon Window for KDE4, LXDE, etc.
            move_x = maxX - (self.GetSize().GetWidth() + 20)
            move_y = maxY - (self.GetSize().GetHeight() + 70)

        self.Move((move_x, move_y))
        self._refreshServerInfo()

    def ShowModal(self, **kwargs):
        self._PyHocaGUI._sub_windows.append(self)
        wx.Dialog.ShowModal(self, **kwargs)

    def _refreshServerInfo(self):

        server_components = self._PyHocaGUI.get_server_components(self.current_profile_name, force=True)
        server_extensions = [ k for k in list(server_components.keys()) if k.startswith('x2goserver-') and k != 'x2goserver-common' ]
        server_extensions.sort()
        server_addons = [ k for k in list(server_components.keys()) if not k.startswith('x2goserver') and k != 'x2goagent' ]
        server_addons.sort()
        server_features = self._PyHocaGUI.get_server_features(self.current_profile_name, force=True)
        halftab = '    '
        newline = '\n'

        self.infoArea.AppendText(_('X2Go Server')+':'+2*newline)
        self.infoArea.AppendText(halftab+_('Server Core')+':'+newline)
        self.infoArea.AppendText(newline)
        self.infoArea.AppendText(2*halftab+'%s (%s)\n' % ('x2goserver', server_components['x2goserver']))
        if 'x2goserver-common' in list(server_components.keys()):
            self.infoArea.AppendText(2*halftab+'%s (%s)\n' % ('x2goserver-common', server_components['x2goserver-common']))
        self.infoArea.AppendText(2*halftab+'%s (%s)\n' % ('x2goagent', server_components['x2goagent']))
        self.infoArea.AppendText('\n')
        if server_extensions:
            self.infoArea.AppendText(halftab+_('Server Extensions')+':'+newline)
            self.infoArea.AppendText(newline)
            for comp in server_extensions:
                self.infoArea.AppendText(2*halftab+'%s (%s)\n' % (comp, server_components[comp]))
        self.infoArea.AppendText('\n')
        if server_addons:
            self.infoArea.AppendText(_('X2Go Server Add-ons')+':'+2*newline)
            for comp in server_addons:
                self.infoArea.AppendText(2*halftab+'%s (%s)\n' % (comp, server_components[comp]))
        self.infoArea.AppendText('\n')
        self.infoArea.AppendText(_('X2Go Server Features')+':'+2*newline)
        for feature in server_features:
            self.infoArea.AppendText(2*halftab+'%s\n' % (feature))
        self.infoArea.ShowPosition(0)

    def OnRefreshServerInfo(self, evt):
        """\
        Gets called if the Refresh button gets pressed.

        @param evt: event
        @type evt: C{obj}

        """
        self.infoArea.Clear()
        self._refreshServerInfo()

    def OnCancel(self, evt):
        """\
        Continue here, if the user clicks the Cancel button in the dialog box.

        @param evt: event
        @type evt: C{obj}

        """
        self.Hide()
        self.cancel = True

    def Hide(self):
        """\
        When hiding the server info box, remove it from the list of open windows in the main application instance.

        """
        try:
            self._PyHocaGUI._sub_windows.remove(self)
        except (AttributeError, ValueError):
            pass
        self.Show(False)

    def Close(self):
        """\
        Do some PyHocaGUI specific cleanup if this window gets destroyed.

        """
        try:
            self._PyHocaGUI._sub_windows.remove(self)
        except ValueError:
            pass
        try:
            self._PyHocaGUI._temp_disabled_profile_names.remove(self.current_profile_name)
        except ValueError:
            pass
        wx.Dialog.Close(self)
        wx.Dialog.Destroy(self)



# -*- coding: utf-8 -*-
# vim:fenc=utf-8

# Copyright (C) 2010-2020 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program; if not, write to the
# Free Software Foundation, Inc.,
# 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.

# Contributors to the code of this programme:
#     Dick Kniep <dick.kniep@lindix.nl>


import wx
import os

# PyHoca-GUI modules
from . import basepath

# X2Go modules
from x2go.defaults import CURRENT_LOCAL_USER as _CURRENT_LOCAL_USER

class PyHoca_MessageWindow(wx.Dialog):
    """\
    A simple message window for L{PyHocaGUI}.

    """
    def __init__(self, _PyHocaGUI, parent=None, title=None, shortmsg=None, msg=None, icon=None, buttontype='ok',
                 profile_name=None,
                 session_name=None):
        """\
        L{PyHocaGUI} message window.

        @param _PyHocaGUI: main application instance
        @type _PyHocaGUI: C{obj}
        @param parent: the parent (calling) object
        @type parent: C{obj}
        @param title: window title
        @type title: C{str}
        @param shortmsg: a short string that refers to a pre-defined message (hard-coded in this class)
        @type shortmsg: C{str}
        @param msg: the message to be shown in this message box (alternative to C{shortmsg})
        @type msg: C{str}
        @param icon: icon name for an icon to be shown left of the text in this message box
        @type icon: C{str}
        @param buttontype: button types can be: C{ok}, C{okcancel}, C{cancelok}, C{yesno}, and C{noyes}
        @type buttontype: C{str}
        @param profile_name: session profile name of the profile this message box refers to
        @type profile_name: C{str}
        @param session_name: X2Go session name of the session this message box refers to
        @type session_name: C{str}

        """
        self._PyHocaGUI = _PyHocaGUI

        try: wx.EndBusyCursor()
        except: pass

        self._pyhoca_messages = {
            'REALLY_DELETE_PROFILE': _('Are you really sure you want to\ndelete the session profile ,,%s\'\'?') % profile_name,
            'ALREADY_RUNNING': _('{appname} is already running for user ,,{username}\'\'!\n\nOnly one instance of {appname} can be started per\nuser. The {appname} icon can be found in your desktop\'s\nnotification area/systray.').format(appname=self._PyHocaGUI.appname, username=_CURRENT_LOCAL_USER)
        }


        if shortmsg is None:
            show_message = msg
        elif shortmsg in list(self._pyhoca_messages.keys()):
            show_message = self._pyhoca_messages[shortmsg]
        else:
            show_message = 'No message has been given...'

        self.show_message = show_message
        self.result = None

        try:
            if parent is None:
                parent = self._PyHocaGUI.about
        except AttributeError:
            pass

        wx.Dialog.__init__(self, parent, wx.ID_ANY, )
        self.SetTitle('%s' % title)

        _icons_location = basepath.icons_basepath
        icon_folder = 'PyHoca'
        if os.path.isdir(os.path.join(_icons_location, _PyHocaGUI.appname)):
            icon_folder = _PyHocaGUI.appname

        if icon:
            path_to_icon = os.path.normpath('%s/%s/64x64/%s.png' % (_icons_location, icon_folder, icon))
            self.icon = wx.StaticBitmap(self, wx.ID_ANY, wx.Bitmap(path_to_icon, wx.BITMAP_TYPE_ANY))
        self.message = wx.StaticText(self, wx.ID_ANY, self.show_message, size=(-1, -1), style=wx.ALIGN_LEFT)

        btnSizer = wx.BoxSizer(wx.HORIZONTAL)
        msgSizer = wx.BoxSizer(wx.HORIZONTAL)
        mainSizer = wx.BoxSizer(wx.VERTICAL)

        if buttontype in ('yesno', 'noyes'):
            self.yesBtn = wx.Button(self, wx.ID_ANY, _('Yes'),)
            self.noBtn = wx.Button(self, wx.ID_ANY, _('No'))

            self.Bind(wx.EVT_BUTTON, self.OnTrue, self.yesBtn)
            self.Bind(wx.EVT_BUTTON, self.OnFalse, self.noBtn)

            btnSizer.Add(self.yesBtn, flag=wx.ALL, border=5)
            btnSizer.Add(self.noBtn, flag=wx.ALL, border=5)

            if buttontype == 'yesno':
                self.yesBtn.SetDefault()
                self.yesBtn.SetFocus()
            elif buttontype == 'noyes':
                self.noBtn.SetDefault()
                self.noBtn.SetFocus()

        if buttontype in ('ok', 'okcancel', 'cancelok'):

            self.okBtn = wx.Button(self, wx.ID_ANY, _('Ok'),)
            self.Bind(wx.EVT_BUTTON, self.OnTrue, self.okBtn)
            btnSizer.Add(self.okBtn, flag=wx.ALL, border=5)

        if buttontype in ('okcancel', 'cancelok'):

            self.cancelBtn = wx.Button(self, wx.ID_ANY, _('Cancel'))
            self.Bind(wx.EVT_BUTTON, self.OnFalse, self.cancelBtn)
            btnSizer.Add(self.cancelBtn, flag=wx.ALL, border=5)

        if buttontype in ('ok', 'okcancel'):
            self.okBtn.SetDefault()
            self.okBtn.SetFocus()

        if buttontype == 'cancelok':
            self.cancelBtn.SetDefault()
            self.cancelBtn.SetFocus()

        if icon:
            msgSizer.Add(self.icon, flag=wx.RIGHT, border=15)
        msgSizer.Add(self.message, flag=wx.ALL, border=0)
        mainSizer.Add(msgSizer, flag=wx.ALL, border=10)
        mainSizer.Add(btnSizer, flag=wx.ALL|wx.ALIGN_RIGHT, border=5)

        self.SetSizerAndFit(mainSizer)
        self.Layout()

        maxX, maxY = wx.GetDisplaySize()

        self.Move((maxX/2 - self.GetSize().GetWidth()/2, maxY/2 - self.GetSize().GetHeight()/2))
        try:
            self._PyHocaGUI._sub_windows.append(self)
        except AttributeError:
            pass

    def OnTrue(self, evt):
        """\
        Gets called if the user clicks on ,,Yes'' or ,,Ok''.

        @param evt: event
        @type evt: C{obj}

        """
        self.result = True
        self.Hide()

    def OnFalse(self, evt):
        """\
        Gets called if the user clicks on ,,No'' or ,,Cancel''.

        @param evt: event
        @type evt: C{obj}

        """
        self.result = False
        self.Hide()

    def Ok(self):
        """\
        Evaluate the result what the user chose in the message window.

        @return: C{True}, if ,,Ok'' had been chosen
        @rtype: C{bool}

        """
        return self.Yes()

    def Cancel(self):
        """\
        Evaluate the result what the user chose in the message window.

        @return: C{True}, if ,,Cancel'' had been chosen
        @rtype: C{bool}

        """
        return not self.No()

    def Yes(self):
        """\
        Evaluate the result what the user chose in the message window.

        @return: C{True}, if ,,Yes'' had been chosen
        @rtype: C{bool}

        """
        return self.result

    def No(self):
        """\
        Evaluate the result what the user chose in the message window.

        @return: C{True}, if ,,No'' had been chosen
        @rtype: C{bool}

        """
        return not self.Yes()

    def Hide(self):
        """\
        When hiding the message box, remove it from the list of open windows in the main application instance.

        """
        try:
            self._PyHocaGUI._sub_windows.remove(self)
        except AttributeError:
            pass
        self.Show(False)


class PyHoca_MessageWindow_Ok(PyHoca_MessageWindow):
    """\
    A simple ,,Ok'' message window for L{PyHocaGUI}.

    """
    def __init__(self, _PyHocaGUI, parent=None, title=None, shortmsg=None, msg=None, icon='session_warning', **kwargs):
        """\
        @param _PyHocaGUI: main application instance
        @type _PyHocaGUI: C{obj}
        @param parent: the parent (calling) object
        @type parent: C{obj}
        @param title: window title
        @type title: C{str}
        @param shortmsg: a short string that refers to a pre-defined message (hard-coded in this class)
        @type shortmsg: C{str}
        @param msg: the message to be shown in this message box (alternative to C{shortmsg})
        @type msg: C{str}
        @param icon: icon name for an icon to be shown left of the text in this message box
        @type icon: C{str}
        @param kwargs: any other optional argument (will be ignored)
        @type kwargs: C{dict}

        """
        PyHoca_MessageWindow.__init__(self, _PyHocaGUI, parent=parent, title=title, shortmsg=shortmsg, msg=msg, icon=icon, buttontype='ok', **kwargs)


class PyHoca_MessageWindow_OkCancel(PyHoca_MessageWindow):
    """\
    A simple ,,Ok+Cancel'' (default: Ok) message window for L{PyHocaGUI}.

    """
    def __init__(self, _PyHocaGUI, parent=None, title=None, shortmsg=None, msg=None, icon='session_warning', **kwargs):
        """\
        @param _PyHocaGUI: main application instance
        @type _PyHocaGUI: C{obj}
        @param parent: the parent (calling) object
        @type parent: C{obj}
        @param title: window title
        @type title: C{str}
        @param shortmsg: a short string that refers to a pre-defined message (hard-coded in this class)
        @type shortmsg: C{str}
        @param msg: the message to be shown in this message box (alternative to C{shortmsg})
        @type msg: C{str}
        @param icon: icon name for an icon to be shown left of the text in this message box
        @type icon: C{str}
        @param kwargs: any other optional argument (will be ignored)
        @type kwargs: C{dict}

        """
        PyHoca_MessageWindow.__init__(self, _PyHocaGUI, parent=parent, title=title, shortmsg=shortmsg, msg=msg,  icon=icon, buttontype='okcancel', **kwargs)


class PyHoca_MessageWindow_CancelOk(PyHoca_MessageWindow):
    """\
    A simple ,,Ok+Cancel'' (default: Cancel) message window for L{PyHocaGUI}.

    """
    def __init__(self, _PyHocaGUI, parent=None, title=None, shortmsg=None, msg=None, icon='session_warning', **kwargs):
        """\
        @param _PyHocaGUI: main application instance
        @type _PyHocaGUI: C{obj}
        @param parent: the parent (calling) object
        @type parent: C{obj}
        @param title: window title
        @type title: C{str}
        @param shortmsg: a short string that refers to a pre-defined message (hard-coded in this class)
        @type shortmsg: C{str}
        @param msg: the message to be shown in this message box (alternative to C{shortmsg})
        @type msg: C{str}
        @param icon: icon name for an icon to be shown left of the text in this message box
        @type icon: C{str}
        @param kwargs: any other optional argument (will be ignored)
        @type kwargs: C{dict}

        """
        PyHoca_MessageWindow.__init__(self, _PyHocaGUI, parent=parent, title=title, shortmsg=shortmsg, msg=msg, icon=icon, buttontype='cancelok', **kwargs)


class PyHoca_MessageWindow_YesNo(PyHoca_MessageWindow):
    """\
    A simple ,,Yes+No'' (default: Yes) message window for L{PyHocaGUI}.

    """
    def __init__(self, _PyHocaGUI, parent=None, title=None, shortmsg=None, msg=None, icon='session_warning', **kwargs):
        """\
        @param _PyHocaGUI: main application instance
        @type _PyHocaGUI: C{obj}
        @param parent: the parent (calling) object
        @type parent: C{obj}
        @param title: window title
        @type title: C{str}
        @param shortmsg: a short string that refers to a pre-defined message (hard-coded in this class)
        @type shortmsg: C{str}
        @param msg: the message to be shown in this message box (alternative to C{shortmsg})
        @type msg: C{str}
        @param icon: icon name for an icon to be shown left of the text in this message box
        @type icon: C{str}
        @param kwargs: any other optional argument (will be ignored)
        @type kwargs: C{dict}

        """
        PyHoca_MessageWindow.__init__(self, _PyHocaGUI, parent=parent, title=title, shortmsg=shortmsg, msg=msg, icon=icon, buttontype='yesno', **kwargs)

class PyHoca_MessageWindow_NoYes(PyHoca_MessageWindow):
    """\
    A simple ,,Yes+No'' (default: No) message window for L{PyHocaGUI}.

    """
    def __init__(self, _PyHocaGUI, parent=None, title=None, shortmsg=None, msg=None, icon='session_warning', **kwargs):
        """\
        @param _PyHocaGUI: main application instance
        @type _PyHocaGUI: C{obj}
        @param parent: the parent (calling) object
        @type parent: C{obj}
        @param title: window title
        @type title: C{str}
        @param shortmsg: a short string that refers to a pre-defined message (hard-coded in this class)
        @type shortmsg: C{str}
        @param msg: the message to be shown in this message box (alternative to C{shortmsg})
        @type msg: C{str}
        @param icon: icon name for an icon to be shown left of the text in this message box
        @type icon: C{str}
        @param kwargs: any other optional argument (will be ignored)
        @type kwargs: C{dict}

        """
        PyHoca_MessageWindow.__init__(self, _PyHocaGUI, parent=parent, title=title, shortmsg=shortmsg, msg=msg, icon=icon, buttontype='noyes', **kwargs)

